from typing import Callable, ClassVar, Literal, Mapping, Sequence, TypeVar
from numpy.random import RandomState
from ..metrics import get_scorer_names as get_scorer_names
from scipy import optimize as optimize
from ..model_selection._split import BaseShuffleSplit
from ..utils.extmath import softmax as softmax, row_norms as row_norms
from joblib import effective_n_jobs as effective_n_jobs
from ..metrics import get_scorer as get_scorer
from ..preprocessing import (
    LabelEncoder as LabelEncoder,
    LabelBinarizer as LabelBinarizer,
)
from ._glm.glm import NewtonCholeskySolver as NewtonCholeskySolver
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import check_is_fitted as check_is_fitted
from numpy import ndarray
from ..utils._param_validation import StrOptions as StrOptions, Interval as Interval
from numbers import Integral as Integral, Real as Real
from ._sag import sag_solver as sag_solver
from ..model_selection import check_cv as check_cv
from ..utils import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
    compute_class_weight as compute_class_weight,
    check_random_state as check_random_state,
)
from ._base import LinearClassifierMixin, SparseCoefMixin, BaseEstimator
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from .._loss.loss import (
    HalfBinomialLoss as HalfBinomialLoss,
    HalfMultinomialLoss as HalfMultinomialLoss,
)
from ._linear_loss import LinearModelLoss as LinearModelLoss
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..model_selection import BaseCrossValidator

LogisticRegressionCV_Self = TypeVar(
    "LogisticRegressionCV_Self", bound="LogisticRegressionCV"
)
LogisticRegression_Self = TypeVar("LogisticRegression_Self", bound="LogisticRegression")


# Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
#         Fabian Pedregosa <f@bianp.net>
#         Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
#         Manoj Kumar <manojkumarsivaraj334@gmail.com>
#         Lars Buitinck
#         Simon Wu <s8wu@uwaterloo.ca>
#         Arthur Mensch <arthur.mensch@m4x.org

import numbers
import warnings

import numpy as np


_LOGISTIC_SOLVER_CONVERGENCE_MSG: str = ...


class LogisticRegression(LinearClassifierMixin, SparseCoefMixin, BaseEstimator):
    n_iter_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        penalty: Literal["l1", "l2", "elasticnet", "l2"] | None = "l2",
        *,
        dual: bool = False,
        tol: Float = 1e-4,
        C: Float = 1.0,
        fit_intercept: bool = True,
        intercept_scaling: Float = 1,
        class_weight: None | Mapping | str = None,
        random_state: RandomState | None | Int = None,
        solver: Literal[
            "lbfgs", "liblinear", "newton-cg", "newton-cholesky", "sag", "saga", "lbfgs"
        ] = "lbfgs",
        max_iter: Int = 100,
        multi_class: Literal["auto", "ovr", "multinomial", "auto"] = "auto",
        verbose: Int = 0,
        warm_start: bool = False,
        n_jobs: None | Int = None,
        l1_ratio: None | Float = None,
    ) -> None:
        ...

    def fit(
        self: LogisticRegression_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LogisticRegression_Self:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...


class LogisticRegressionCV(LogisticRegression, LinearClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: ndarray = ...
    l1_ratio_: ndarray = ...
    C_: ndarray = ...
    scores_: dict = ...
    coefs_paths_: ndarray = ...
    l1_ratios_: ndarray = ...
    Cs_: ndarray = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    for param in ["C", "warm_start", "l1_ratio"]:
        pass

    def __init__(
        self,
        *,
        Cs: Sequence[float] | int = 10,
        fit_intercept: bool = True,
        cv: int | None | BaseShuffleSplit | BaseCrossValidator = None,
        dual: bool = False,
        penalty: Literal["l1", "l2", "elasticnet", "l2"] = "l2",
        scoring: None | str | Callable = None,
        solver: Literal[
            "lbfgs", "liblinear", "newton-cg", "newton-cholesky", "sag", "saga", "lbfgs"
        ] = "lbfgs",
        tol: Float = 1e-4,
        max_iter: Int = 100,
        class_weight: None | Mapping | str = None,
        n_jobs: None | Int = None,
        verbose: Int = 0,
        refit: bool = True,
        intercept_scaling: Float = 1.0,
        multi_class: Literal["auto", "ovr", "multinomial", "auto"] = "auto",
        random_state: RandomState | None | Int = None,
        l1_ratios: None | Sequence[float] = None,
    ) -> None:
        ...

    def fit(
        self: LogisticRegressionCV_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> LogisticRegressionCV_Self:
        ...

    def score(
        self, X: MatrixLike, y: ArrayLike, sample_weight: None | ArrayLike = None
    ) -> float:
        ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

###############################################################################
# These fast normal calculation routines are adapted from mne-python

def _fast_cross_3d(x, y): ...
def _calculate_normals(rr, tris): ...
def resize(image: NDArray, shape: tuple, kind: str = "linear") -> NDArray: ...



from ...classes.digraph import DiGraph
from ...classes.multidigraph import MultiDiGraph
from ...utils.decorators import not_implemented_for

__all__ = [
    "number_attracting_components",
    "attracting_components",
    "is_attracting_component",
]

def attracting_components(G: DiGraph | MultiDiGraph): ...
def number_attracting_components(G: DiGraph | MultiDiGraph) -> int: ...
def is_attracting_component(G: DiGraph | MultiDiGraph) -> bool: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize.cython_optimize._zeros, version: unspecified
import typing
import builtins as _mod_builtins

EXAMPLES_MAP: dict
__doc__: str
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
__test__: dict
def full_output_example() -> typing.Any:
    '\n    Example of Cython optimize zeros functions with full output.\n\n    Parameters\n    ----------\n    args : sequence of float\n        extra arguments of zero function\n    xa : float\n        first boundary of zero function\n    xb : float\n        second boundary of zero function\n    xtol : float\n        absolute tolerance of zero function\n    rtol : float\n        relative tolerance of zero function\n    mitr : int\n        max. iteration of zero function\n\n    Returns\n    -------\n    full_output : dict\n        the root, number of function calls, number of iterations, and the zero\n        function error number \n\n    This example finds the roots of a 3rd order polynomial with coefficients\n    given as `args`.\n    '
    ...

def loop_example() -> typing.Any:
    '\n    Example of Cython optimize zeros functions with map.\n\n    Parameters\n    ----------\n    method : str\n        name of the Cython optimize zeros function to call\n    a0 : sequence of float\n        first extra argument which is mapped to output\n    args : sequence of float\n        the remaining extra arguments which are constant\n    xa : float\n        first bound of zero function\n    xb : float\n        second bound of zero function\n    xtol : float\n        absolute tolerance of zero function\n    rtol : float\n        relative tolerance of zero function\n    mitr : int\n        max. iteration of zero function\n\n    Returns\n    -------\n    roots : sequence of float\n        the root for each of the values of `a0`\n\n    This example finds the roots of a 3rd order polynomial given a sequence of\n    constant terms as `a0` and fixed 1st, 2nd, and 3rd order terms in `args`.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, List, Optional, Sequence

from django.apps.config import AppConfig

def check_generic_foreign_keys(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> List[Any]: ...
def check_model_name_lengths(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> List[Any]: ...



from typing import Any, ClassVar, Iterable, Literal, Sequence, TypeVar
from scipy import sparse as sparse
from .utils.validation import (
    check_memory as check_memory,
    check_is_fitted as check_is_fitted,
)
from itertools import islice as islice
from .base import BaseEstimator
from joblib import Memory
from .utils.metaestimators import available_if as available_if, _BaseComposition
from pandas.core.frame import DataFrame
from scipy.sparse import spmatrix
from .utils._bunch import Bunch
from collections import defaultdict as defaultdict
from .base import clone as clone, TransformerMixin
from numpy import ndarray
from .exceptions import NotFittedError as NotFittedError
from ._typing import Int, MatrixLike, ArrayLike, Float
from .preprocessing import FunctionTransformer as FunctionTransformer
from pandas.core.series import Series
from .utils import check_pandas_support as check_pandas_support
from .utils.parallel import delayed as delayed, Parallel as Parallel

FeatureUnion_Self = TypeVar("FeatureUnion_Self", bound="FeatureUnion")
Pipeline_Self = TypeVar("Pipeline_Self", bound="Pipeline")


import numpy as np

__all__ = ["Pipeline", "FeatureUnion", "make_pipeline", "make_union"]


class Pipeline(_BaseComposition):

    # BaseEstimator interface
    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        steps: Sequence[tuple],
        *,
        memory: None | Memory | str = None,
        verbose: bool = False
    ) -> None:
        ...

    def set_output(
        self, *, transform: None | Literal["default", "pandas"] = None
    ) -> BaseEstimator:
        ...

    def get_params(self, deep: bool = True) -> dict[str, Any]:
        ...

    def set_params(self: Pipeline_Self, **kwargs) -> Pipeline_Self:
        ...

    def __len__(self) -> int:
        ...

    def __getitem__(self, ind: slice | int):
        ...

    @property
    def named_steps(self) -> Bunch:
        ...

    def fit(
        self: Pipeline_Self,
        X: list[str] | ndarray | Iterable | DataFrame,
        y: list[Int] | list[int] | Iterable | None | Series | ndarray = None,
        **fit_params
    ) -> Pipeline_Self:
        ...

    def fit_transform(
        self, X: Iterable, y: Iterable | Series | None | ndarray = None, **fit_params
    ) -> ndarray:
        ...

    def predict(
        self, X: list[str] | ndarray | Iterable | DataFrame, **predict_params
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def fit_predict(
        self, X: Iterable, y: Iterable | None = None, **fit_params
    ) -> ndarray:
        ...

    def predict_proba(
        self, X: Iterable | ndarray | DataFrame, **predict_proba_params
    ) -> ndarray:
        ...

    def decision_function(self, X: Iterable | ndarray | DataFrame) -> ndarray:
        ...

    def score_samples(self, X: Iterable) -> ndarray:
        ...

    def predict_log_proba(self, X: Iterable, **predict_log_proba_params) -> ndarray:
        ...

    def transform(self, X: Iterable | ndarray | DataFrame) -> ndarray:
        ...

    def inverse_transform(self, Xt: MatrixLike) -> ndarray:
        ...

    def score(
        self,
        X: list[str] | ndarray | Iterable | DataFrame,
        y: Iterable | Series | None | ndarray = None,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    @property
    def classes_(self) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    @property
    def n_features_in_(self) -> int:
        ...

    @property
    def feature_names_in_(self) -> ndarray:
        ...

    def __sklearn_is_fitted__(self) -> bool:
        ...


def make_pipeline(
    *steps, memory: None | Memory | str = None, verbose: bool = False
) -> Pipeline:
    ...


class FeatureUnion(TransformerMixin, _BaseComposition):

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        transformer_list: Sequence[tuple[str, TransformerMixin | Pipeline]],
        *,
        n_jobs: None | Int = None,
        transformer_weights: None | dict = None,
        verbose: bool = False
    ) -> None:
        ...

    def set_output(
        self, *, transform: None | Literal["default", "pandas"] = None
    ) -> BaseEstimator:
        ...

    @property
    def named_transformers(self) -> Bunch:
        ...

    def get_params(self, deep: bool = True) -> dict[str, Any]:
        ...

    def set_params(self: FeatureUnion_Self, **kwargs) -> FeatureUnion_Self:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def fit(
        self: FeatureUnion_Self,
        X: Iterable | ArrayLike,
        y: None | MatrixLike = None,
        **fit_params
    ) -> FeatureUnion_Self:
        ...

    def fit_transform(
        self,
        X: Iterable | ArrayLike | DataFrame,
        y: Series | None | MatrixLike = None,
        **fit_params
    ) -> ndarray | spmatrix:
        ...

    def transform(self, X: Iterable | ArrayLike | DataFrame) -> ndarray | spmatrix:
        ...

    @property
    def n_features_in_(self) -> int:
        ...

    def __sklearn_is_fitted__(self):
        ...


def make_union(
    *transformers, n_jobs: None | Int = None, verbose: bool = False
) -> FeatureUnion:
    ...



from typing import (
    Any,
    Callable,
)

from pandas._typing import Scalar

def make_rolling_apply(
    func: Callable[..., Scalar],
    args: tuple,
    nogil: bool,
    parallel: bool,
    nopython: bool,
): ...
def generate_numba_apply_func(
    args: tuple,
    kwargs: dict[str, Any],
    func: Callable[..., Scalar],
    engine_kwargs: dict[str, bool] | None,
): ...



from typing import Any, ClassVar, TypeVar
from warnings import warn as warn
from numpy.random import RandomState
from ..base import BaseEstimator
from ..utils.random import sample_without_replacement as sample_without_replacement
from ..metrics import r2_score as r2_score, accuracy_score as accuracy_score
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import (
    has_fit_parameter as has_fit_parameter,
    check_is_fitted as check_is_fitted,
)
from abc import ABCMeta, abstractmethod
from ..tree import (
    DecisionTreeClassifier as DecisionTreeClassifier,
    DecisionTreeRegressor as DecisionTreeRegressor,
)
from numpy import ndarray
from ..utils._param_validation import (
    Interval as Interval,
    HasMethods as HasMethods,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from functools import partial as partial
from ..base import ClassifierMixin, RegressorMixin
from ..utils import (
    check_random_state as check_random_state,
    column_or_1d as column_or_1d,
    indices_to_mask as indices_to_mask,
)
from ._base import BaseEnsemble
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..utils.metaestimators import available_if as available_if
from .._typing import MatrixLike, ArrayLike, Int

BaseBagging_Self = TypeVar("BaseBagging_Self", bound="BaseBagging")


# Author: Gilles Louppe <g.louppe@gmail.com>
# License: BSD 3 clause


import itertools
import numbers
import numpy as np


__all__ = ["BaggingClassifier", "BaggingRegressor"]

MAX_INT = ...


class BaseBagging(BaseEnsemble, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        estimator=None,
        n_estimators: int = 10,
        *,
        max_samples: float = 1.0,
        max_features: float = 1.0,
        bootstrap: bool = True,
        bootstrap_features: bool = False,
        oob_score: bool = False,
        warm_start: bool = False,
        n_jobs=None,
        random_state=None,
        verbose: int = 0,
        base_estimator: str = "deprecated",
    ) -> None:
        ...

    def fit(
        self: BaseBagging_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaggingRegressor | BaseBagging_Self:
        ...

    def estimators_samples_(self):
        ...


class BaggingClassifier(ClassifierMixin, BaseBagging):
    oob_decision_function_: ndarray = ...
    oob_score_: float = ...
    n_classes_: list | int = ...
    classes_: ndarray = ...
    estimators_features_: list[ArrayLike] = ...
    estimators_samples_: list[ArrayLike] = ...
    estimators_: list[BaseEstimator] = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    base_estimator_: BaseEstimator = ...
    estimator_: BaseEstimator = ...

    def __init__(
        self,
        estimator: Any = None,
        n_estimators: Int = 10,
        *,
        max_samples: float | int = 1.0,
        max_features: float | int = 1.0,
        bootstrap: bool = True,
        bootstrap_features: bool = False,
        oob_score: bool = False,
        warm_start: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class BaggingRegressor(RegressorMixin, BaseBagging):
    oob_prediction_: ndarray = ...
    oob_score_: float = ...
    estimators_features_: list[ArrayLike] = ...
    estimators_samples_: list[ArrayLike] = ...
    estimators_: list[BaseEstimator] = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    base_estimator_: BaseEstimator = ...
    estimator_: BaseEstimator = ...

    def __init__(
        self,
        estimator: Any = None,
        n_estimators: Int = 10,
        *,
        max_samples: float | int = 1.0,
        max_features: float | int = 1.0,
        bootstrap: bool = True,
        bootstrap_features: bool = False,
        oob_score: bool = False,
        warm_start: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import os
from os import path as op
import numpy as np

from .wavefront import WavefrontReader, WavefrontWriter
from .stl import load_stl

def read_mesh(
    fname: str,
) -> tuple[np.ndarray, np.ndarray | None, np.ndarray, np.ndarray | None]: ...
def write_mesh(
    fname: str,
    vertices: ArrayLike,
    faces: ArrayLike | None,
    normals: ArrayLike,
    texcoords: ArrayLike | None,
    name: str = "",
    format: str | None = None,
    overwrite: bool = False,
    reshape_faces: bool = True,
): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from ._dict_learning import (
    dict_learning as dict_learning,
    MiniBatchDictionaryLearning as MiniBatchDictionaryLearning,
)
from ..linear_model import ridge_regression as ridge_regression
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from ..utils.extmath import svd_flip as svd_flip
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import check_random_state as check_random_state
from ..utils.validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
)

_BaseSparsePCA_Self = TypeVar("_BaseSparsePCA_Self", bound="_BaseSparsePCA")


import numpy as np


class _BaseSparsePCA(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | int = None,
        *,
        alpha: int = 1,
        ridge_alpha: float = 0.01,
        max_iter: int = 1000,
        tol: float = 1e-8,
        method: str = "lars",
        n_jobs=None,
        verbose: bool = False,
        random_state=None,
    ) -> None:
        ...

    def fit(
        self: _BaseSparsePCA_Self, X: MatrixLike, y: Any = None
    ) -> _BaseSparsePCA_Self | MiniBatchSparsePCA:
        ...

    def transform(self, X: ArrayLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...


class SparsePCA(_BaseSparsePCA):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    mean_: ndarray = ...
    n_iter_: int = ...
    n_components_: int = ...
    error_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        alpha: Float = 1,
        ridge_alpha: Float = 0.01,
        max_iter: Int = 1000,
        tol: Float = 1e-8,
        method: Literal["lars", "cd", "lars"] = "lars",
        n_jobs: None | Int = None,
        U_init: None | MatrixLike = None,
        V_init: None | MatrixLike = None,
        verbose: int | bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class MiniBatchSparsePCA(_BaseSparsePCA):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    mean_: ndarray = ...
    n_iter_: int = ...
    n_components_: int = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        alpha: Int = 1,
        ridge_alpha: Float = 0.01,
        n_iter: str | Int = "deprecated",
        max_iter: None | Int = None,
        callback: None | Callable = None,
        batch_size: Int = 3,
        verbose: int | bool = False,
        shuffle: bool = True,
        n_jobs: None | Int = None,
        method: Literal["lars", "cd", "lars"] = "lars",
        random_state: RandomState | None | Int = None,
        tol: Float = 1e-3,
        max_no_improvement: None | int = 10,
    ) -> None:
        ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division as division

from .application import Application as Application  # noqa
from ._default_app import (
    use_app as use_app,
    create as create,
    run as run,
    quit as quit,
    process_events as process_events,
)  # noqa
from .canvas import (
    Canvas as Canvas,
    MouseEvent as MouseEvent,
    KeyEvent as KeyEvent,
)  # noqa
from .timer import Timer as Timer  # noqa
from . import base as base  # noqa



from typing import ClassVar, TypeVar
from numpy.random import RandomState
from scipy.special import binom as binom
from scipy import linalg as linalg
from itertools import combinations as combinations
from ._base import LinearModel
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils._param_validation import Interval as Interval
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from scipy.linalg.lapack import get_lapack_funcs as get_lapack_funcs
from joblib import effective_n_jobs as effective_n_jobs
from ..base import RegressorMixin
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import Int, Float, ArrayLike
from ..utils import check_random_state as check_random_state

TheilSenRegressor_Self = TypeVar("TheilSenRegressor_Self", bound="TheilSenRegressor")


# Author: Florian Wilhelm <florian.wilhelm@gmail.com>
#
# License: BSD 3 clause


import warnings

import numpy as np

_EPSILON = ...


class TheilSenRegressor(RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_subpopulation_: int = ...
    n_iter_: int = ...
    breakdown_: float = ...
    intercept_: float = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_subpopulation: float | Int = 1e4,
        n_subsamples: None | Int = None,
        max_iter: Int = 300,
        tol: Float = 1.0e-3,
        random_state: RandomState | None | Int = None,
        n_jobs: None | Int = None,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: TheilSenRegressor_Self, X: ArrayLike, y: ArrayLike
    ) -> TheilSenRegressor_Self:
        ...



from heapq import heappop, heappush
from itertools import count

from ...classes.graph import Graph

__all__ = ["astar_path", "astar_path_length"]

def astar_path(G: Graph, source, target, heuristic=None, weight="weight"): ...
def astar_path_length(G: Graph, source, target, heuristic=None, weight="weight"): ...



from typing import Any, Optional

from django.contrib.gis.geometry import hex_regex as hex_regex  # noqa: F401
from django.contrib.gis.geometry import json_regex as json_regex
from django.contrib.gis.geometry import wkt_regex as wkt_regex
from django.contrib.gis.geos.base import GEOSBase as GEOSBase
from django.contrib.gis.geos.mutable_list import ListMixin as ListMixin

class GEOSGeometryBase(GEOSBase):
    ptr_type: Any = ...
    destructor: Any = ...
    has_cs: bool = ...
    __class__: Any = ...
    def __init__(self, ptr: Any, cls: Any) -> None: ...
    def __copy__(self) -> Any: ...
    def __deepcopy__(self, memodict: Any) -> Any: ...
    @staticmethod
    def from_ewkt(ewkt: Any) -> Any: ...
    @classmethod
    def from_gml(cls, gml_string: Any) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...
    def __or__(self, other: Any) -> Any: ...
    def __and__(self, other: Any) -> Any: ...
    def __sub__(self, other: Any) -> Any: ...
    def __xor__(self, other: Any) -> Any: ...
    @property
    def coord_seq(self) -> Any: ...
    @property
    def geom_type(self) -> Any: ...
    @property
    def geom_typeid(self) -> Any: ...
    @property
    def num_geom(self) -> Any: ...
    @property
    def num_coords(self) -> Any: ...
    @property
    def num_points(self) -> Any: ...
    @property
    def dims(self) -> Any: ...
    def normalize(self) -> None: ...
    @property
    def empty(self) -> Any: ...
    @property
    def hasz(self) -> Any: ...
    @property
    def ring(self) -> Any: ...
    @property
    def simple(self) -> Any: ...
    @property
    def valid(self) -> Any: ...
    @property
    def valid_reason(self) -> Any: ...
    def contains(self, other: Any) -> Any: ...
    def covers(self, other: Any) -> Any: ...
    def crosses(self, other: Any) -> Any: ...
    def disjoint(self, other: Any) -> Any: ...
    def equals(self, other: Any) -> Any: ...
    def equals_exact(self, other: Any, tolerance: int = ...) -> Any: ...
    def intersects(self, other: Any) -> Any: ...
    def overlaps(self, other: Any) -> Any: ...
    def relate_pattern(self, other: Any, pattern: Any) -> Any: ...
    def touches(self, other: Any) -> Any: ...
    def within(self, other: Any) -> Any: ...
    @property
    def srid(self) -> Any: ...
    @srid.setter
    def srid(self, srid: Any) -> None: ...
    @property
    def ewkt(self) -> Any: ...
    @property
    def wkt(self) -> Any: ...
    @property
    def hex(self) -> Any: ...
    @property
    def hexewkb(self) -> Any: ...
    @property
    def json(self) -> Any: ...
    geojson: Any = ...
    @property
    def wkb(self) -> Any: ...
    @property
    def ewkb(self) -> Any: ...
    @property
    def kml(self) -> Any: ...
    @property
    def prepared(self) -> Any: ...
    @property
    def ogr(self) -> Any: ...
    @property
    def srs(self) -> Any: ...
    @property
    def crs(self) -> Any: ...
    ptr: Any = ...
    def transform(self, ct: Any, clone: bool = ...) -> Any: ...
    @property
    def boundary(self) -> Any: ...
    def buffer(self, width: Any, quadsegs: int = ...) -> Any: ...
    def buffer_with_style(
        self,
        width: Any,
        quadsegs: int = ...,
        end_cap_style: int = ...,
        join_style: int = ...,
        mitre_limit: float = ...,
    ) -> Any: ...
    @property
    def centroid(self) -> Any: ...
    @property
    def convex_hull(self) -> Any: ...
    def difference(self, other: Any) -> Any: ...
    @property
    def envelope(self) -> Any: ...
    def intersection(self, other: Any) -> Any: ...
    @property
    def point_on_surface(self) -> Any: ...
    def relate(self, other: Any) -> Any: ...
    def simplify(
        self, tolerance: float = ..., preserve_topology: bool = ...
    ) -> Any: ...
    def sym_difference(self, other: Any) -> Any: ...
    @property
    def unary_union(self) -> Any: ...
    def union(self, other: Any) -> Any: ...
    @property
    def area(self) -> Any: ...
    def distance(self, other: Any) -> Any: ...
    @property
    def extent(self) -> Any: ...
    @property
    def length(self) -> Any: ...
    def clone(self) -> Any: ...

class LinearGeometryMixin:
    def interpolate(self, distance: Any) -> Any: ...
    def interpolate_normalized(self, distance: Any) -> Any: ...
    def project(self, point: Any) -> Any: ...
    def project_normalized(self, point: Any) -> Any: ...
    @property
    def merged(self) -> Any: ...
    @property
    def closed(self) -> Any: ...

class GEOSGeometry(GEOSGeometryBase, ListMixin):
    srid: Any = ...
    def __init__(self, geo_input: Any, srid: Optional[Any] = ...) -> None: ...



"""
This type stub file was generated by pyright.
"""

"""Sentinel class for constants with useful reprs"""
class Sentinel:
    def __init__(self, name, module, docstring=...) -> None:
        ...
    
    def __repr__(self):
        ...
    





import itertools as it
import math
from collections import defaultdict, namedtuple

from ...classes.graph import Graph
from ...utils import not_implemented_for, py_random_state

__all__ = ["k_edge_augmentation", "is_k_edge_connected", "is_locally_k_edge_connected"]

def is_k_edge_connected(G: Graph, k) -> bool: ...
def is_locally_k_edge_connected(G: Graph, s, t, k) -> bool: ...
def k_edge_augmentation(
    G: Graph, k, avail=None, weight: str | None = None, partial: bool = False
): ...
def partial_k_edge_augmentation(G: Graph, k, avail, weight: str | None = None): ...
def one_edge_augmentation(
    G: Graph, avail=None, weight: str | None = None, partial: bool = False
): ...
def bridge_augmentation(G: Graph, avail=None, weight: str | None = None): ...

# --- Algorithms and Helpers ---

MetaEdge = ...

def unconstrained_one_edge_augmentation(G: Graph): ...
def weighted_one_edge_augmentation(
    G: Graph, avail, weight: str | None = None, partial: bool = False
): ...
def unconstrained_bridge_augmentation(G: Graph): ...
def weighted_bridge_augmentation(G: Graph, avail, weight: str | None = None): ...
def collapse(G: Graph, grouped_nodes): ...
def complement_edges(G: Graph): ...
@py_random_state(4)
def greedy_k_edge_augmentation(
    G: Graph, k, avail=None, weight: str | None = None, seed=None
): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.integrate._quadpack, version:  1.13 
import typing
import builtins as _mod_builtins
import quadpack as _mod_quadpack

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__version__: str
def _qagie(fun, bound, inf, args, full_output, epsabs, epsrel, limit) -> typing.Any:
    '[result,abserr,infodict,ier] = _qagie(fun, bound, inf, | args, full_output, epsabs, epsrel, limit)'
    ...

def _qagpe(fun, a, b, points, args, full_output, epsabs, epsrel, limit) -> typing.Any:
    '[result,abserr,infodict,ier] = _qagpe(fun, a, b, points, | args, full_output, epsabs, epsrel, limit)'
    ...

def _qagse(fun, a, b, args, full_output, epsabs, epsrel, limit) -> typing.Any:
    '[result,abserr,infodict,ier] = _qagse(fun, a, b, | args, full_output, epsabs, epsrel, limit)'
    ...

def _qawce(fun, a, b, c, args, full_output, epsabs, epsrel, limit) -> typing.Any:
    '[result,abserr,infodict,ier] = _qawce(fun, a, b, c, | args, full_output, epsabs, epsrel, limit)'
    ...

def _qawfe(fun, a, omega, integr, args, full_output, epsabs, limlst, limit, maxp1) -> typing.Any:
    '[result,abserr,infodict,ier] = _qawfe(fun, a, omega, integr, | args, full_output, epsabs, limlst, limit, maxp1)'
    ...

def _qawoe(fun, a, b, omega, integr, args, full_output, epsabs, epsrel, limit, maxp1, icall, momcom, chebmo) -> typing.Any:
    '[result,abserr,infodict,ier] = _qawoe(fun, a, b, omega, integr, | args, full_output, epsabs, epsrel, limit, maxp1, icall, momcom, chebmo)'
    ...

def _qawse(fun, a, b, alfa, beta) -> typing.Any:
    '[result,abserr,infodict,ier] = _qawse(fun, a, b, (alfa, beta), integr, | args, full_output, epsabs, epsrel, limit)'
    ...

error = _mod_quadpack.error
def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""Payload system for IPython.

Authors:

* Fernando Perez
* Brian Granger
"""
class PayloadManager(Configurable):
    _payload = ...
    def write_payload(self, data, single=...): # -> None:
        """Include or update the specified `data` payload in the PayloadManager.

        If a previous payload with the same source exists and `single` is True,
        it will be overwritten with the new one.
        """
        ...
    
    def read_payload(self): # -> List:
        ...
    
    def clear_payload(self): # -> None:
        ...
    





from numpy.typing import NDArray, ArrayLike
import numpy as np

from scipy.interpolate import interp1d
from scipy.constants import golden_ratio
from scipy.fft import fft, ifft, fftfreq, fftshift
from ._warps import warp
from .._shared.utils import convert_to_float
from warnings import warn
from functools import partial

__all__ = ["radon", "order_angles_golden_ratio", "iradon", "iradon_sart"]

def radon(
    image, theta=None, circle: bool = True, *, preserve_range: bool = False
) -> NDArray: ...
def _sinogram_circle_to_square(sinogram): ...
def _get_fourier_filter(size, filter_name): ...
def iradon(
    radon_image: ArrayLike,
    theta=None,
    output_size: int | None = None,
    filter_name: str = "ramp",
    interpolation: str = "linear",
    circle: bool = True,
    preserve_range: bool = True,
): ...
def order_angles_golden_ratio(theta): ...
def iradon_sart(
    radon_image,
    theta=None,
    image=None,
    projection_shifts=None,
    clip=None,
    relaxation: float = 0.15,
    dtype=None,
) -> NDArray: ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------
import copy
from .style import Style

namespace: str = ...

class Element(object):
    def __init__(self, content=None, parent=None): ...
    @property
    def root(self): ...
    @property
    def parent(self): ...
    @property
    def style(self): ...
    @property
    def viewport(self): ...



from ..classes.digraph import DiGraph
from collections import defaultdict

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = ["prefix_tree", "random_tree", "prefix_tree_recursive"]

def prefix_tree(paths) -> DiGraph: ...
def prefix_tree_recursive(paths) -> DiGraph: ...

# From the Wikipedia article on Prüfer sequences:
#
# > Generating uniformly distributed random Prüfer sequences and
# > converting them into the corresponding trees is a straightforward
# > method of generating uniformly distributed random labelled trees.
#
@py_random_state(1)
def random_tree(n: int, seed=None, create_using=None): ...



from typing import Type
from matplotlib._typing import *
from matplotlib.transforms import Transform
from matplotlib.text import Text
from matplotlib.backend_bases import _Backend, FigureCanvasBase, FigureManagerBase, GraphicsContextBase
from matplotlib.font_manager import FontProperties
from matplotlib.transforms import Affine2DBase

from enum import Enum

from traitlets import Int
from . import _backend_pdf_ps

backend_version: str = ...
debugPS: bool = ...

class PsBackendHelper:
    def __init__(self) -> None: ...

ps_backend_helper: PsBackendHelper = ...
papersize: dict[str, tuple[float, float] ]= ...

def quote_ps_string(s: str) -> str: ...

class RendererPS(_backend_pdf_ps.RendererPDFPSBase):
    def __init__(self, width, height, pswriter, imagedpi=...) -> None: ...
    def set_color(
        self, r: float, g: float, b: float, store: float | bool = ...
    ) -> None: ...
    def set_linewidth(self, linewidth: float, store: bool = ...) -> None: ...
    def set_linejoin(self, linejoin: str, store: bool = ...) -> None: ...
    def set_linecap(self, linecap: str, store: bool = ...) -> None: ...
    def set_linedash(self, offset: int, seq: None, store: bool = ...) -> None: ...
    def set_font(self, fontname: str, fontsize: float, store: bool = ...) -> None: ...
    def create_hatch(self, hatch)-> str: ...
    def get_image_magnification(self) -> float: ...
    def draw_image(
        self,
        gc: GraphicsContextBase,
        x: Scalar,
        y: Scalar,
        im,
        transform: Affine2DBase = ...,
    )-> None: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def draw_markers(
        self,
        gc: GraphicsContextBase,
        marker_path,
        marker_trans: Transform,
        path,
        trans: Transform,
        rgbFace=...,
    )-> None: ...
    def draw_path_collection(
        self,
        gc,
        master_transform,
        paths,
        all_transforms,
        offsets,
        offsetTrans,
        facecolors,
        edgecolors,
        linewidths,
        linestyles,
        antialiaseds,
        urls,
        offset_position,
    )-> None: ...
    def draw_tex(self, gc, x, y, s, prop, angle, *, mtext=...)-> None: ...
    def draw_text(
        self,
        gc: GraphicsContextBase,
        x: float,
        y: float,
        s: str,
        prop: FontProperties,
        angle: float,
        ismath=...,
        mtext: Text = ...,
    )-> None: ...
    def draw_mathtext(self, gc, x, y, s, prop, angle) -> None: ...
    def draw_gouraud_triangle(self, gc: GraphicsContextBase, points, colors, trans)-> None: ...
    def draw_gouraud_triangles(self, gc, points, colors, trans)-> None: ...

class _Orientation(Enum):
    def swap_if_landscape(self, shape: tuple[float, int]) -> tuple[float, int]: ...

class FigureCanvasPS(FigureCanvasBase):
    fixed_dpi: Int = ...
    filetypes: dict[str, str] = ...
    def get_default_filetype(self)-> str: ...

    def print_ps(self, outfile, *args,
            metadata=None, papertype=None, orientation='portrait',
            **kwargs)-> None: ...
    def print_eps(self, outfile, *args,
            metadata=None, papertype=None, orientation='portrait',
            **kwargs)-> None: ...
    def draw(self)-> None: ...

def convert_psfrags(
    tmpfile,
    psfrags,
    font_preamble,
    custom_preamble,
    paper_width,
    paper_height,
    orientation,
)-> bool: ...
def gs_distill(tmpfile, eps=..., ptype=..., bbox=..., rotated=...) -> None: ...
def xpdf_distill(tmpfile, eps=..., ptype=..., bbox=..., rotated=...) -> None: ...
def get_bbox_header(lbrt, rotated=...) -> tuple[str, str]: ...
def pstoeps(tmpfile, bbox=..., rotated=...)-> None: ...

FigureManagerPS: Type[FigureManagerBase] = ...
psDefs: list[str] = ...

class _BackendPS(_Backend):
    FigureCanvas = FigureCanvasPS



from collections import defaultdict as defaultdict
import platform
import argparse

SCIPY_MIN_VERSION: str = ...
JOBLIB_MIN_VERSION: str = ...
THREADPOOLCTL_MIN_VERSION: str = ...
PYTEST_MIN_VERSION: str = ...
CYTHON_MIN_VERSION: str = ...


# 'build' and 'install' is included to have structured metadata for CI.
# It will NOT be included in setup's extras_require
# The values are (version_spec, comma separated tags)
dependent_packages: dict = ...


# create inverse mapping for setuptools
tag_to_packages: dict = ...



def parse_date_time(date_col, time_col): ...
def parse_date_fields(year_col, month_col, day_col): ...
def parse_all_fields(
    year_col, month_col, day_col, hour_col, minute_col, second_col
): ...
def generic_parser(parse_func, *cols): ...



StsOk = 0
StsBackTrace = -1
StsError = -2
StsInternal = -3
StsNoMem = -4
StsBadArg = -5
StsBadFunc = -6
StsNoConv = -7
StsAutoTrace = -8
HeaderIsNull = -9
BadImageSize = -10
BadOffset = -11
BadDataPtr = -12
BadStep = -13
BadModelOrChSeq = -14
BadNumChannels = -15
BadNumChannel1U = -16
BadDepth = -17
BadAlphaChannel = -18
BadOrder = -19
BadOrigin = -20
BadAlign = -21
BadCallBack = -22
BadTileSize = -23
BadCOI = -24
BadROISize = -25
MaskIsTiled = -26
StsNullPtr = -27
StsVecLengthErr = -28
StsFilterStructContentErr = -29
StsKernelStructContentErr = -30
StsFilterOffsetErr = -31
StsBadSize = -201
StsDivByZero = -202
StsInplaceNotSupported = -203
StsObjectNotFound = -204
StsUnmatchedFormats = -205
StsBadFlag = -206
StsBadPoint = -207
StsBadMask = -208
StsUnmatchedSizes = -209
StsUnsupportedFormat = -210
StsOutOfRange = -211
StsParseError = -212
StsNotImplemented = -213
StsBadMemBlock = -214
StsAssert = -215
GpuNotSupported = -216
GpuApiCallError = -217
OpenGlNotSupported = -218
OpenGlApiCallError = -219
OpenCLApiCallError = -220
OpenCLDoubleNotSupported = -221
OpenCLInitError = -222
OpenCLNoAMDBlasFft = -223


STS_OK = 0
STS_BACK_TRACE = -1
STS_ERROR = -2
STS_INTERNAL = -3
STS_NO_MEM = -4
STS_BAD_ARG = -5
STS_BAD_FUNC = -6
STS_NO_CONV = -7
STS_AUTO_TRACE = -8
HEADER_IS_NULL = -9
BAD_IMAGE_SIZE = -10
BAD_OFFSET = -11
BAD_DATA_PTR = -12
BAD_STEP = -13
BAD_MODEL_OR_CH_SEQ = -14
BAD_NUM_CHANNELS = -15
BAD_NUM_CHANNEL1U = -16
BAD_DEPTH = -17
BAD_ALPHA_CHANNEL = -18
BAD_ORDER = -19
BAD_ORIGIN = -20
BAD_ALIGN = -21
BAD_CALL_BACK = -22
BAD_TILE_SIZE = -23
BAD_COI = -24
BAD_ROISIZE = -25
MASK_IS_TILED = -26
STS_NULL_PTR = -27
STS_VEC_LENGTH_ERR = -28
STS_FILTER_STRUCT_CONTENT_ERR = -29
STS_KERNEL_STRUCT_CONTENT_ERR = -30
STS_FILTER_OFFSET_ERR = -31
STS_BAD_SIZE = -201
STS_DIV_BY_ZERO = -202
STS_INPLACE_NOT_SUPPORTED = -203
STS_OBJECT_NOT_FOUND = -204
STS_UNMATCHED_FORMATS = -205
STS_BAD_FLAG = -206
STS_BAD_POINT = -207
STS_BAD_MASK = -208
STS_UNMATCHED_SIZES = -209
STS_UNSUPPORTED_FORMAT = -210
STS_OUT_OF_RANGE = -211
STS_PARSE_ERROR = -212
STS_NOT_IMPLEMENTED = -213
STS_BAD_MEM_BLOCK = -214
STS_ASSERT = -215
GPU_NOT_SUPPORTED = -216
GPU_API_CALL_ERROR = -217
OPEN_GL_NOT_SUPPORTED = -218
OPEN_GL_API_CALL_ERROR = -219
OPEN_CLAPI_CALL_ERROR = -220
OPEN_CLDOUBLE_NOT_SUPPORTED = -221
OPEN_CLINIT_ERROR = -222
OPEN_CLNO_AMDBLAS_FFT = -223



from vispy.util.svg.color import Color
from typing import Literal
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

from .mesh import MeshVisual

class HistogramVisual(MeshVisual):
    def __init__(
        self,
        data: ArrayLike,
        bins: ArrayLike | int = 10,
        color: Color | str = "w",
        orientation: Literal["h", "v"] = "h",
    ): ...



from numpy.typing import ArrayLike

import os
import re
import struct
import threading
import functools
from tempfile import NamedTemporaryFile

import numpy as np
from numpy import testing
from numpy.testing import (
    assert_array_equal,
    assert_array_almost_equal,
    assert_array_less,
    assert_array_almost_equal_nulp,
    assert_equal,
    TestCase,
    assert_allclose,
    assert_almost_equal,
    assert_,
    assert_warns,
    assert_no_warnings,
)

import warnings

from ..data._fetchers import _fetch
from ._warnings import expected_warnings

SKIP_RE = ...

skipif = ...
xfail = ...
parametrize = ...
raises = ...
fixture = ...

# true if python is running in 32bit mode
# Calculate the size of a void * pointer in bits
# https://docs.python.org/3/library/struct.html
arch32 = ...

_error_on_warnings = ...

def assert_less(a, b, msg=None): ...
def assert_greater(a, b, msg=None): ...
def doctest_skip_parser(func): ...
def roundtrip(image, plugin, suffix): ...
def color_check(plugin, fmt="png"): ...
def mono_check(plugin, fmt="png"): ...
def setup_test(): ...
def teardown_test(): ...
def fetch(data_filename): ...
def test_parallel(num_threads: int = 2, warnings_matching: ArrayLike | None = None): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.fft._pocketfft.pypocketfft, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
def c2c(a, axes=..., forward=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    "c2c(a: array, axes: object = None, forward: bool = True, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a complex FFT.\n\nParameters\n----------\na : numpy.ndarray (any complex or real type)\n    The input data. If its type is real, a more efficient real-to-complex\n    transform will be used.\naxes : list of integers\n    The axes along which the FFT is carried out.\n    If not set, all axes will be transformed.\nforward : bool\n    If `True`, a negative sign is used in the exponent, else a positive one.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the product of the lengths of the transformed axes.\nout : numpy.ndarray (same shape as `a`, complex type with same accuracy as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape as `a`, complex type with same accuracy as `a`)\n    The transformed data.\n\n"
    ...

def c2r(a, axes=..., lastsize=..., forward=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    'c2r(a: array, axes: object = None, lastsize: int = 0, forward: bool = True, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms an FFT whose output is strictly real.\n\nParameters\n----------\na : numpy.ndarray (any complex type)\n    The input data\naxes : list of integers\n    The axes along which the FFT is carried out.\n    If not set, all axes will be transformed in ascending order.\nlastsize : the output size of the last axis to be transformed.\n    If the corresponding input axis has size n, this can be 2*n-2 or 2*n-1.\nforward : bool\n    If `True`, a negative sign is used in the exponent, else a positive one.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the product of the lengths of the transformed output axes.\nout : numpy.ndarray (real type with same accuracy as `a`)\n    For the required shape, see the `Returns` section.\n    Must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (real type with same accuracy as `a`)\n    The transformed data. The shape is identical to that of the input array,\n    except for the axis that was transformed last, which has now `lastsize`\n    entries.\n\n'
    ...

def dct(a, type, axes=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    "dct(a: array, type: int, axes: object = None, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a discrete cosine transform.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\ntype : integer\n    the type of DCT. Must be in [1; 4].\naxes : list of integers\n    The axes along which the transform is carried out.\n    If not set, all axes will be transformed.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : make transform orthogonal and divide by sqrt(N)\n      2 : divide by N\n    where N is the product of n_i for every transformed axis i.\n    n_i is 2*(<axis_length>-1 for type 1 and 2*<axis length>\n    for types 2, 3, 4.\n    Making the transform orthogonal involves the following additional steps\n    for every 1D sub-transform:\n      Type 1 : multiply first and last input value by sqrt(2)\n               divide first and last output value by sqrt(2)\n      Type 2 : divide first output value by sqrt(2)\n      Type 3 : multiply first input value by sqrt(2)\n      Type 4 : nothing\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data\n\n"
    ...

def dst(a, type, axes=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    "dst(a: array, type: int, axes: object = None, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a discrete sine transform.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\ntype : integer\n    the type of DST. Must be in [1; 4].\naxes : list of integers\n    The axes along which the transform is carried out.\n    If not set, all axes will be transformed.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : make transform orthogonal and divide by sqrt(N)\n      2 : divide by N\n    where N is the product of n_i for every transformed axis i.\n    n_i is 2*(<axis_length>+1 for type 1 and 2*<axis length>\n    for types 2, 3, 4.\n    Making the transform orthogonal involves the following additional steps\n    for every 1D sub-transform:\n      Type 1 : nothing\n      Type 2 : divide first output value by sqrt(2)\n      Type 3 : multiply first input value by sqrt(2)\n      Type 4 : nothing\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data\n\n"
    ...

def genuine_hartley(a, axes=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    "genuine_hartley(a: array, axes: object = None, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a full Hartley transform.\nA full Fourier transform is carried out over the requested axes, and the\nsum of real and imaginary parts of the result is stored in the output\narray. For a single transformed axis, this is identical to `separable_hartley`,\nbut when transforming multiple axes, the results are different.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\naxes : list of integers\n    The axes along which the transform is carried out.\n    If not set, all axes will be transformed.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the product of the lengths of the transformed axes.\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data\n\n"
    ...

def good_size() -> typing.Any:
    'Returns a good length to pad an FFT to.\n\nParameters\n----------\ntarget : int\n    Minimum transform length\nreal : bool, optional\n    True if either input or output of FFT should be fully real.\n\nReturns\n-------\nout : int\n    The smallest fast size >= n\n\n'
    ...

def r2c(a, axes=..., forward=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    'r2c(a: array, axes: object = None, forward: bool = True, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms an FFT whose input is strictly real.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\naxes : list of integers\n    The axes along which the FFT is carried out.\n    If not set, all axes will be transformed in ascending order.\nforward : bool\n    If `True`, a negative sign is used in the exponent, else a positive one.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the product of the lengths of the transformed input axes.\nout : numpy.ndarray (complex type with same accuracy as `a`)\n    For the required shape, see the `Returns` section.\n    Must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (complex type with same accuracy as `a`)\n    The transformed data. The shape is identical to that of the input array,\n    except for the axis that was transformed last. If the length of that axis\n    was n on input, it is n//2+1 on output.\n\n'
    ...

def r2r_fftpack(a, axes, real2hermitian, forward, inorm=..., out=..., nthreads=...) -> typing.Any:
    "r2r_fftpack(a: array, axes: object, real2hermitian: bool, forward: bool, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a real-valued FFT using the FFTPACK storage scheme.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\naxes : list of integers\n    The axes along which the FFT is carried out.\n    If not set, all axes will be transformed.\nreal2hermitian : bool\n    if True, the input is purely real and the output will have Hermitian\n    symmetry and be stored in FFTPACK's halfcomplex ordering, otherwise the\n    opposite.\nforward : bool\n    If `True`, a negative sign is used in the exponent, else a positive one.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the length of `axis`.\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data. The shape is identical to that of the input array.\n\n"
    ...

def separable_hartley(a, axes=..., inorm=..., out=..., nthreads=...) -> typing.Any:
    "separable_hartley(a: array, axes: object = None, inorm: int = 0, out: object = None, nthreads: int = 1) -> array\n\nPerforms a separable Hartley transform.\nFor every requested axis, a 1D forward Fourier transform is carried out, and\nthe real and imaginary parts of the result are added before the next axis is\nprocessed.\n\nParameters\n----------\na : numpy.ndarray (any real type)\n    The input data\naxes : list of integers\n    The axes along which the transform is carried out.\n    If not set, all axes will be transformed.\ninorm : int\n    Normalization type\n      0 : no normalization\n      1 : divide by sqrt(N)\n      2 : divide by N\n    where N is the product of the lengths of the transformed axes.\nout : numpy.ndarray (same shape and data type as `a`)\n    May be identical to `a`, but if it isn't, it must not overlap with `a`.\n    If None, a new array is allocated to store the output.\nnthreads : int\n    Number of threads to use. If 0, use the system default (typically governed\n    by the `OMP_NUM_THREADS` environment variable).\n\nReturns\n-------\nnumpy.ndarray (same shape and data type as `a`)\n    The transformed data\n\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Mapping
from numpy.typing import NDArray, ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

def _fix_colors(colors): ...

class MeshData(object):
    def __init__(
        self,
        vertices: NDArray | None = None,
        faces: NDArray | None = None,
        edges=None,
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        vertex_values: NDArray | None = None,
    ): ...
    def get_faces(self): ...
    def get_edges(self, indexed: str | None = None) -> NDArray: ...
    def set_faces(self, faces: NDArray): ...
    def get_vertices(self, indexed: str | None = None) -> NDArray: ...
    def get_bounds(self) -> ArrayLike: ...
    def set_vertices(
        self,
        verts: NDArray | None = None,
        indexed: str | None = None,
        reset_normals: bool = True,
    ): ...
    def reset_normals(self): ...
    def has_face_indexed_data(self): ...
    def has_edge_indexed_data(self): ...
    def has_vertex_color(self): ...
    def has_vertex_value(self): ...
    def has_face_color(self): ...
    def get_face_normals(self, indexed: str | None = None) -> NDArray: ...
    def get_vertex_normals(self, indexed: str | None = None) -> NDArray: ...
    def get_vertex_colors(self, indexed: str | None = None) -> NDArray: ...
    def get_vertex_values(self, indexed: str | None = None) -> NDArray: ...
    def set_vertex_colors(self, colors: ArrayLike, indexed: str | None = None): ...
    def set_vertex_values(self, values: ArrayLike, indexed: str | None = None): ...
    def get_face_colors(self, indexed: str | None = None) -> NDArray: ...
    def set_face_colors(self, colors: ArrayLike, indexed: str | None = None): ...
    @property
    def n_faces(self): ...
    @property
    def n_vertices(self): ...
    def get_edge_colors(self): ...
    def _compute_unindexed_vertices(self): ...
    def get_vertex_faces(self): ...
    def _compute_edges(self, indexed=None): ...
    def save(self) -> Mapping: ...
    def restore(self, state: Mapping): ...
    def is_empty(self): ...



from numpy.typing import NDArray
from skimage.feature import multiscale_basic_features

class TrainableSegmenter(object):
    def __init__(self, clf=None, features_func=None): ...
    def compute_features(self, image): ...
    def fit(self, image: NDArray, labels: NDArray): ...
    def predict(self, image: NDArray): ...

def fit_segmenter(labels: NDArray, features: NDArray, clf): ...
def predict_segmenter(features: NDArray, clf) -> NDArray: ...



from typing import Callable

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import inspect as inspect

from .random import random as random
from .circular import circular as circular
from .force_directed import fruchterman_reingold as fruchterman_reingold
from .networkx_layout import NetworkxCoordinates as NetworkxCoordinates

_layout_map: dict = ...

AVAILABLE_LAYOUTS = ...

def get_layout(name: str, *args, **kwargs) -> Callable: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class

"""Implementation of magic functions for the extension machinery.
"""
@magics_class
class ExtensionMagics(Magics):
    """Magics to manage the IPython extensions system."""
    @line_magic
    def load_ext(self, module_str): # -> None:
        """Load an IPython extension by its module name."""
        ...
    
    @line_magic
    def unload_ext(self, module_str): # -> None:
        """Unload an IPython extension by its module name.

        Not all extensions can be unloaded, only those which define an
        ``unload_ipython_extension`` function.
        """
        ...
    
    @line_magic
    def reload_ext(self, module_str): # -> None:
        """Reload an IPython extension by its module name."""
        ...
    





from numpy.typing import NDArray, ArrayLike
import numpy as np

def hough_line_peaks(
    hspace,
    angles,
    dists,
    min_distance: int = 9,
    min_angle: int = 10,
    threshold: float | None = None,
    num_peaks: int = ...,
) -> tuple[NDArray, ...]: ...
def hough_circle(image, radius, normalize=True, full_output=False): ...
def hough_ellipse(
    image,
    threshold: int = 4,
    accuracy=1,
    min_size: int = 4,
    max_size: int | None = None,
): ...
def hough_line(image, theta=None): ...
def probabilistic_hough_line(
    image,
    threshold: int = 10,
    line_length: int = 50,
    line_gap: int = 10,
    theta=None,
    seed: int | None = None,
) -> ArrayLike: ...
def hough_circle_peaks(
    hspaces,
    radii,
    min_xdistance: int = 1,
    min_ydistance: int = 1,
    threshold: float | None = None,
    num_peaks: int = ...,
    total_num_peaks: int = ...,
    normalize: bool = False,
) -> tuple[NDArray, ...]: ...
def label_distant_points(
    xs: ArrayLike,
    ys: ArrayLike,
    min_xdistance: int,
    min_ydistance: int,
    max_points: int,
) -> ArrayLike: ...



from .backend_bases import RendererBase
from typing import Sequence
from ._typing import *
from .figure import Figure, FigureBase, SubplotParams

class GridSpecBase:
    def __init__(
        self,
        nrows: int,
        ncols: int,
        height_ratios: ArrayLike = ...,
        width_ratios: ArrayLike = ...,
    ) -> None: ...
    def __repr__(self) -> str: ...

    nrows = ...
    ncols = ...
    def get_geometry(self) -> tuple[int, int]: ...
    def get_subplot_params(self, figure: Figure = ...): ...
    def new_subplotspec(
        self, loc: tuple[int, int], rowspan: int = ..., colspan: int = ...
    ) -> SubplotSpec: ...
    def set_width_ratios(self, width_ratios: ArrayLike) -> None: ...
    def get_width_ratios(self) -> list[float]: ...
    def set_height_ratios(self, height_ratios: ArrayLike) -> None: ...
    def get_height_ratios(self) -> list[float]: ...
    def get_grid_positions(self, fig: Figure, raw: bool = False): ...
    def __getitem__(self, key) -> SubplotSpec: ...
    def subplots(self, *, sharex=..., sharey=..., squeeze=..., subplot_kw=...): ...

class GridSpec(GridSpecBase):
    def __init__(
        self,
        nrows: int,
        ncols: int,
        figure: FigureBase = ...,
        left: float = ...,
        bottom: float = ...,
        right: float = ...,
        top: float = ...,
        wspace: float = ...,
        hspace: float = ...,
        width_ratios: ArrayLike = ...,
        height_ratios: ArrayLike = ...,
    ) -> None: ...
    def update(self, **kwargs) -> None: ...
    def get_subplot_params(self, figure: FigureBase = ...) -> SubplotParams: ...
    def locally_modified_subplot_params(self) -> list[str]: ...
    def tight_layout(
        self,
        figure: Figure,
        renderer: RendererBase = ...,
        pad: float = ...,
        h_pad: float = ...,
        w_pad: float = ...,
        rect: Sequence[float] = ...,
    ) -> None: ...

class GridSpecFromSubplotSpec(GridSpecBase):
    def __init__(
        self,
        nrows: int,
        ncols: int,
        subplot_spec: SubplotSpec,
        wspace: float = ...,
        hspace: float = ...,
        height_ratios: ArrayLike = ...,
        width_ratios: ArrayLike = ...,
    ) -> None: ...
    def get_subplot_params(self, figure: Figure = ...) -> SubplotParams: ...
    def get_topmost_subplotspec(self) -> SubplotSpec: ...

class SubplotSpec:
    def __init__(
        self,
        gridspec: GridSpecBase,
        num1: int,
        num2: int = ...,
    ) -> None: ...
    def __repr__(self) -> str: ...
    @property
    def num2(self): ...
    @num2.setter
    def num2(self, value): ...
    def get_gridspec(self) -> GridSpecBase: ...
    def get_geometry(self): ...
    @property
    def rowspan(self) -> range: ...
    @property
    def colspan(self) -> range: ...
    def is_first_row(self) -> bool: ...
    def is_last_row(self) -> bool: ...
    def is_first_col(self) -> bool: ...
    def is_last_col(self) -> bool: ...
    def get_position(self, figure: Figure): ...
    def get_topmost_subplotspec(self) -> SubplotSpec: ...
    def __eq__(self, other: SubplotSpec) -> bool: ...
    def __hash__(self) -> int: ...
    def subgridspec(
        self, nrows: int, ncols: int, **kwargs
    ) -> GridSpecFromSubplotSpec: ...



"""
This type stub file was generated by pyright.
"""

import os
import signal
import sys
import warnings
from dataclasses import dataclass
from typing import Any, Callable, List, Optional
from prompt_toolkit.application.current import get_app
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.key_binding.key_processor import KeyPressEvent
from prompt_toolkit.key_binding.bindings import named_commands as nc
from prompt_toolkit.key_binding.bindings.completion import display_completions_like_readline
from prompt_toolkit.key_binding.vi_state import InputMode, ViState
from prompt_toolkit.filters import Condition
from IPython.core.getipython import get_ipython
from IPython.terminal.shortcuts import auto_match as match, auto_suggest
from IPython.terminal.shortcuts.filters import filter_from_string
from IPython.utils.decorators import undoc
from prompt_toolkit.enums import DEFAULT_BUFFER
from IPython.core.error import TryNext
from IPython.lib.clipboard import ClipboardEmpty, tkinter_clipboard_get, win32_clipboard_get

"""
Module to define and register Terminal IPython shortcuts with
:mod:`prompt_toolkit`
"""
__all__ = ["create_ipython_shortcuts"]
@dataclass
class BaseBinding:
    command: Callable[[KeyPressEvent], Any]
    keys: List[str]
    ...


@dataclass
class RuntimeBinding(BaseBinding):
    filter: Condition
    ...


@dataclass
class Binding(BaseBinding):
    condition: Optional[str] = ...
    def __post_init__(self): # -> None:
        ...
    


def create_identifier(handler: Callable):
    ...

AUTO_MATCH_BINDINGS = ...
AUTO_SUGGEST_BINDINGS = ...
SIMPLE_CONTROL_BINDINGS = ...
ALT_AND_COMOBO_CONTROL_BINDINGS = ...
def add_binding(bindings: KeyBindings, binding: Binding): # -> None:
    ...

def create_ipython_shortcuts(shell, skip=...) -> KeyBindings:
    """Set up the prompt_toolkit keyboard shortcuts for IPython.

    Parameters
    ----------
    shell: InteractiveShell
        The current IPython shell Instance
    skip: List[Binding]
        Bindings to skip.

    Returns
    -------
    KeyBindings
        the keybinding instance for prompt toolkit.

    """
    ...

def reformat_and_execute(event): # -> None:
    """Reformat code and execute it"""
    ...

def reformat_text_before_cursor(buffer, document, shell): # -> None:
    ...

def handle_return_or_newline_or_execute(event): # -> None:
    ...

def newline_or_execute_outer(shell): # -> (event: Unknown) -> None:
    ...

def previous_history_or_previous_completion(event): # -> None:
    """
    Control-P in vi edit mode on readline is history next, unlike default prompt toolkit.

    If completer is open this still select previous completion.
    """
    ...

def next_history_or_next_completion(event): # -> None:
    """
    Control-N in vi edit mode on readline is history previous, unlike default prompt toolkit.

    If completer is open this still select next completion.
    """
    ...

def dismiss_completion(event): # -> None:
    """Dismiss completion"""
    ...

def reset_buffer(event): # -> None:
    """Reset buffer"""
    ...

def reset_search_buffer(event): # -> None:
    """Reset search buffer"""
    ...

def suspend_to_bg(event): # -> None:
    """Suspend to background"""
    ...

def quit(event): # -> None:
    """
    Quit application with ``SIGQUIT`` if supported or ``sys.exit`` otherwise.

    On platforms that support SIGQUIT, send SIGQUIT to the current process.
    On other platforms, just exit the process with a message.
    """
    ...

def indent_buffer(event): # -> None:
    """Indent buffer"""
    ...

def newline_autoindent(event): # -> None:
    """Insert a newline after the cursor indented appropriately.

    Fancier version of former ``newline_with_copy_margin`` which should
    compute the correct indentation of the inserted line. That is to say, indent
    by 4 extra space after a function definition, class definition, context
    manager... And dedent by 4 space after ``pass``, ``return``, ``raise ...``.
    """
    ...

def open_input_in_editor(event): # -> None:
    """Open code from input in external editor"""
    ...

if sys.platform == "win32":
    @undoc
    def win_paste(event): # -> None:
        ...
    
else:
    ...
KEY_BINDINGS = ...



from math import sqrt
import numpy as np
from scipy import ndimage as ndi

STREL_4 = ...
STREL_8 = ...

# Coefficients from
# Ohser J., Nagel W., Schladitz K. (2002) The Euler Number of Discretized Sets
# - On the Choice of Adjacency in Homogeneous Lattices.
# In: Mecke K., Stoyan D. (eds) Morphology of Condensed Matter. Lecture Notes
# in Physics, vol 600. Springer, Berlin, Heidelberg.
# The value of coefficients correspond to the contributions to the Euler number
# of specific voxel configurations, which are themselves encoded thanks to a
# LUT. Computing the Euler number from the addition of the contributions of
# local configurations is possible thanks to an integral geometry formula
# (see the paper by Ohser et al. for more details).
EULER_COEFS2D_4: list = ...
EULER_COEFS2D_8: list = ...
EULER_COEFS3D_26 = ...

def euler_number(image, connectivity: int | None = None) -> int: ...
def perimeter(image, neighbourhood=4) -> float: ...
def perimeter_crofton(image, directions=4) -> float: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy._lib._uarray._uarray, version: unspecified
import typing
import builtins as _mod_builtins
import uarray as _mod_uarray

BackendNotImplementedError = _mod_uarray.BackendNotImplementedError
_Function = _mod_uarray._Function
_SetBackendContext = _mod_uarray._SetBackendContext
_SkipBackendContext = _mod_uarray._SkipBackendContext
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def clear_all_globals() -> typing.Any:
    ...

def clear_backends() -> typing.Any:
    ...

def register_backend() -> typing.Any:
    ...

def set_global_backend() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.
"""

"""Utility functions for finding modules

Utility functions for finding modules on sys.path.

"""
def find_mod(module_name): # -> None:
    """
    Find module `module_name` on sys.path, and return the path to module `module_name`.

      - If `module_name` refers to a module directory, then return path to __init__ file.
        - If `module_name` is a directory without an __init__file, return None.
      - If module is missing or does not have a `.py` or `.pyw` extension, return None.
        - Note that we are not interested in running bytecode.
      - Otherwise, return the fill path of the module.

    Parameters
    ----------
    module_name : str

    Returns
    -------
    module_path : str
        Path to module `module_name`, its __init__.py, or None,
        depending on above conditions.
    """
    ...




from typing import Literal
from ...base import is_classifier as is_classifier, BaseEstimator
from .._classification import check_consistent_length as check_consistent_length
from matplotlib.axes import Axes
from matplotlib.artist import Artist
from ...utils import check_matplotlib_support as check_matplotlib_support
from ..._typing import ArrayLike, Float, MatrixLike
from .. import (
    average_precision_score as average_precision_score,
    precision_recall_curve as precision_recall_curve,
)
from matplotlib.figure import Figure


class PrecisionRecallDisplay:
    figure_: Figure = ...
    ax_: Axes = ...
    line_: Artist = ...

    def __init__(
        self,
        precision: ArrayLike,
        recall: ArrayLike,
        *,
        average_precision: None | Float = None,
        estimator_name: None | str = None,
        pos_label: None | str | int = None,
    ) -> None:
        ...

    def plot(
        self, ax: None | Axes = None, *, name: None | str = None, **kwargs
    ) -> PrecisionRecallDisplay:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator: BaseEstimator,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        *,
        sample_weight: None | ArrayLike = None,
        pos_label: None | str | int = None,
        response_method: Literal[
            "predict_proba", "decision_function", "auto", "auto"
        ] = "auto",
        name: None | str = None,
        ax: None | Axes = None,
        **kwargs,
    ) -> PrecisionRecallDisplay:
        ...

    @classmethod
    def from_predictions(
        cls,
        y_true: ArrayLike,
        y_pred: ArrayLike,
        *,
        sample_weight: None | ArrayLike = None,
        pos_label: None | str | int = None,
        name: None | str = None,
        ax: None | Axes = None,
        **kwargs,
    ) -> PrecisionRecallDisplay:
        ...



from typing import Mapping

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import io
import os
from os import path as op
import inspect
import json
import sys
import platform
import getopt
import traceback
import tempfile
import atexit
from shutil import rmtree

import numpy as np

from .event import EmitterGroup, EventEmitter, Event
from .logs import logger, set_log_level, use_log_level

file_types: list = ...
file_types = ...

config: None = ...
_data_path: None = ...
_allowed_config_keys: None = ...

def _init(): ...

###############################################################################
# Command line flag parsing

VISPY_HELP: str = ...

def _parse_command_line_arguments(): ...

###############################################################################
# CONFIG

# Adapted from pyzolib/paths.py:
# https://bitbucket.org/pyzo/pyzolib/src/tip/paths.py
def _get_vispy_app_dir(): ...

class ConfigEvent(Event):
    def __init__(self, changes): ...

class Config(object):
    def __init__(self, **kwargs): ...
    def __getitem__(self, item): ...
    def __setitem__(self, item, val): ...
    def _check_key_val(self, key, val): ...
    def update(self, **kwargs): ...
    def __repr__(self): ...

def get_config_keys() -> Mapping: ...
def _get_config_fname(): ...
def _load_config(): ...
def save_config(**kwargs): ...
def set_data_dir(directory: str | None = None, create: bool = False, save: bool = False): ...
def _enable_profiling(): ...

_profiler: None = ...

def _profile_atexit(): ...
def sys_info(fname: str | None = None, overwrite: bool = False) -> str: ...

class _TempDir(str):
    def __new__(self): ...
    def __init__(self): ...
    def cleanup(self): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division
from ... import gloo

class Compiler(object):
    def __init__(self, namespace=None, **shaders): ...
    def __getitem__(self, item): ...
    def compile(self, pretty: bool = True): ...
    def _rename_objects_fast(self): ...
    def _rename_objects_pretty(self): ...
    def _is_global(self, obj): ...
    def _name_available(self, obj, name, shaders): ...
    def _assign_name(self, obj, name, shaders): ...



from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copradiusight (c) 2014, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from ..color import Color
from .polygon import PolygonVisual

class RectangleVisual(PolygonVisual):
    def __init__(
        self,
        center: ArrayLike | None = None,
        color: Color | str = "black",
        border_color: Color | None = None,
        border_width: int = 1,
        height: float = 1.0,
        width: float = 1.0,
        radius: ArrayLike | float = [0.0, 0.0, 0.0, 0.0],
        **kwargs,
    ): ...
    @staticmethod
    def _generate_vertices(center, radius, height, width): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, center): ...
    @property
    def height(self): ...
    @height.setter
    def height(self, height): ...
    @property
    def width(self): ...
    @width.setter
    def width(self, width): ...
    @property
    def radius(self): ...
    @radius.setter
    def radius(self, radius): ...
    def _regen_pos(self): ...




from numpy import float32 as G_H_DTYPE
from numpy import float32 as X_BITSET_INNER_DTYPE
from numpy import float64 as X_DTYPE
from numpy import float64 as Y_DTYPE
from numpy import uint32 as X_BINNED_DTYPE
from numpy import uint8 as X_BINNED_DTYPE_C
import numpy as np


ALMOST_INF: float = 1e300
MonotonicConstraint: int

class PREDICTOR_RECORD_DTYPE:
    value: Y_DTYPE
    count: np.uint32
    feature_idx: np.uint32
    num_threshold: X_DTYPE
    missing_go_to_left: np.uint8
    left: np.uint32
    right: np.uint32
    gain: Y_DTYPE
    depth: np.uint32
    is_leaf: np.uint8
    bin_threshold: X_BINNED_DTYPE
    is_categorical: np.uint8
    bitset_idx: np.uint32




from collections.abc import Iterable
from numpy.typing import ArrayLike
from ..classes.graph import Graph
from ..utils import not_implemented_for, py_random_state

__all__ = ["maximal_independent_set"]

@py_random_state(2)
def maximal_independent_set(
    G: Graph, nodes: ArrayLike | Iterable | None = None, seed=None
) -> ArrayLike: ...



from typing import Any, Optional

from django.db.models import Aggregate
from django.db.models.fields import Field

class GeoAggregate(Aggregate):
    function: Any = ...
    is_extent: bool = ...
    @property
    def output_field(self) -> Field[Any, Any]: ...
    def as_sql(
        self,
        compiler: Any,
        connection: Any,
        function: Optional[Any] = ...,
        **extra_context: Any
    ) -> Any: ...
    def as_oracle(
        self, compiler: Any, connection: Any, **extra_context: Any
    ) -> Any: ...
    def resolve_expression(
        self,
        query: Optional[Any] = ...,
        allow_joins: bool = ...,
        reuse: Optional[Any] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> Any: ...

class Collect(GeoAggregate):
    name: str = ...
    output_field_class: Any = ...

class Extent(GeoAggregate):
    name: str = ...
    def __init__(self, expression: Any, **extra: Any) -> None: ...

class Extent3D(GeoAggregate):
    name: str = ...
    def __init__(self, expression: Any, **extra: Any) -> None: ...

class MakeLine(GeoAggregate):
    name: str = ...
    output_field_class: Any = ...

class Union(GeoAggregate):
    name: str = ...
    output_field_class: Any = ...



"""
This type stub file was generated by pyright.
"""

import sys
from string import Formatter

"""
Utilities for working with strings and text.

Inheritance diagram:

.. inheritance-diagram:: IPython.utils.text
   :parts: 3
"""
if sys.platform == 'win32':
    date_format = ...
else:
    ...
class LSString(str):
    """String derivative with a special access attributes.

    These are normal strings, but with the special attributes:

        .l (or .list) : value as list (split on newlines).
        .n (or .nlstr): original value (the string itself).
        .s (or .spstr): value as whitespace-separated string.
        .p (or .paths): list of path objects (requires path.py package)

    Any values which require transformations are computed only once and
    cached.

    Such strings are very useful to efficiently interact with the shell, which
    typically only understands whitespace-separated options for commands."""
    def get_list(self):
        ...
    
    list = ...
    def get_spstr(self):
        ...
    
    spstr = ...
    def get_nlstr(self): # -> Self@LSString:
        ...
    
    nlstr = ...
    def get_paths(self):
        ...
    
    paths = ...


class SList(list):
    """List derivative with a special access attributes.

    These are normal lists, but with the special attributes:

    * .l (or .list) : value as list (the list itself).
    * .n (or .nlstr): value as a string, joined on newlines.
    * .s (or .spstr): value as a string, joined on spaces.
    * .p (or .paths): list of path objects (requires path.py package)

    Any values which require transformations are computed only once and
    cached."""
    def get_list(self): # -> Self@SList:
        ...
    
    list = ...
    def get_spstr(self):
        ...
    
    spstr = ...
    def get_nlstr(self):
        ...
    
    nlstr = ...
    def get_paths(self):
        ...
    
    paths = ...
    def grep(self, pattern, prune=..., field=...): # -> SList:
        """ Return all strings matching 'pattern' (a regex or callable)

        This is case-insensitive. If prune is true, return all items
        NOT matching the pattern.

        If field is specified, the match must occur in the specified
        whitespace-separated field.

        Examples::

            a.grep( lambda x: x.startswith('C') )
            a.grep('Cha.*log', prune=1)
            a.grep('chm', field=-1)
        """
        ...
    
    def fields(self, *fields): # -> SList:
        """ Collect whitespace-separated fields from string list

        Allows quick awk-like usage of string lists.

        Example data (in var a, created by 'a = !ls -l')::

            -rwxrwxrwx  1 ville None      18 Dec 14  2006 ChangeLog
            drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython

        * ``a.fields(0)`` is ``['-rwxrwxrwx', 'drwxrwxrwx+']``
        * ``a.fields(1,0)`` is ``['1 -rwxrwxrwx', '6 drwxrwxrwx+']``
          (note the joining by space).
        * ``a.fields(-1)`` is ``['ChangeLog', 'IPython']``

        IndexErrors are ignored.

        Without args, fields() just split()'s the strings.
        """
        ...
    
    def sort(self, field=..., nums=...): # -> SList:
        """ sort by specified fields (see fields())

        Example::

            a.sort(1, nums = True)

        Sorts a by second field, in numerical order (so that 21 > 3)

        """
        ...
    


def indent(instr, nspaces=..., ntabs=..., flatten=...): # -> None:
    """Indent a string a given number of spaces or tabstops.

    indent(str,nspaces=4,ntabs=0) -> indent str by ntabs+nspaces.

    Parameters
    ----------
    instr : basestring
        The string to be indented.
    nspaces : int (default: 4)
        The number of spaces to be indented.
    ntabs : int (default: 0)
        The number of tabs to be indented.
    flatten : bool (default: False)
        Whether to scrub existing indentation.  If True, all lines will be
        aligned to the same indentation.  If False, existing indentation will
        be strictly increased.

    Returns
    -------
    str|unicode : string indented by ntabs and nspaces.

    """
    ...

def list_strings(arg):
    """Always return a list of strings, given a string or list of strings
    as input.

    Examples
    --------
    ::

        In [7]: list_strings('A single string')
        Out[7]: ['A single string']

        In [8]: list_strings(['A single string in a list'])
        Out[8]: ['A single string in a list']

        In [9]: list_strings(['A','list','of','strings'])
        Out[9]: ['A', 'list', 'of', 'strings']
    """
    ...

def marquee(txt=..., width=..., mark=...):
    """Return the input string centered in a 'marquee'.

    Examples
    --------
    ::

        In [16]: marquee('A test',40)
        Out[16]: '**************** A test ****************'

        In [17]: marquee('A test',40,'-')
        Out[17]: '---------------- A test ----------------'

        In [18]: marquee('A test',40,' ')
        Out[18]: '                 A test                 '

    """
    ...

ini_spaces_re = ...
def num_ini_spaces(strng):
    """Return the number of initial spaces in a string"""
    ...

def format_screen(strng):
    """Format a string for screen printing.

    This removes some latex-type format codes."""
    ...

def dedent(text):
    """Equivalent of textwrap.dedent that ignores unindented first line.

    This means it will still dedent strings like:
    '''foo
    is a bar
    '''

    For use in wrap_paragraphs.
    """
    ...

def wrap_paragraphs(text, ncols=...):
    """Wrap multiple paragraphs to fit a specified width.

    This is equivalent to textwrap.wrap, but with support for multiple
    paragraphs, as separated by empty lines.

    Returns
    -------
    list of complete paragraphs, wrapped to fill `ncols` columns.
    """
    ...

def strip_email_quotes(text):
    """Strip leading email quotation characters ('>').

    Removes any combination of leading '>' interspersed with whitespace that
    appears *identically* in all lines of the input text.

    Parameters
    ----------
    text : str

    Examples
    --------

    Simple uses::

        In [2]: strip_email_quotes('> > text')
        Out[2]: 'text'

        In [3]: strip_email_quotes('> > text\\n> > more')
        Out[3]: 'text\\nmore'

    Note how only the common prefix that appears in all lines is stripped::

        In [4]: strip_email_quotes('> > text\\n> > more\\n> more...')
        Out[4]: '> text\\n> more\\nmore...'

    So if any line has no quote marks ('>'), then none are stripped from any
    of them ::

        In [5]: strip_email_quotes('> > text\\n> > more\\nlast different')
        Out[5]: '> > text\\n> > more\\nlast different'
    """
    ...

def strip_ansi(source):
    """
    Remove ansi escape codes from text.

    Parameters
    ----------
    source : str
        Source to remove the ansi from
    """
    ...

class EvalFormatter(Formatter):
    """A String Formatter that allows evaluation of simple expressions.

    Note that this version interprets a `:`  as specifying a format string (as per
    standard string formatting), so if slicing is required, you must explicitly
    create a slice.

    This is to be used in templating cases, such as the parallel batch
    script templates, where simple arithmetic on arguments is useful.

    Examples
    --------
    ::

        In [1]: f = EvalFormatter()
        In [2]: f.format('{n//4}', n=8)
        Out[2]: '2'

        In [3]: f.format("{greeting[slice(2,4)]}", greeting="Hello")
        Out[3]: 'll'
    """
    def get_field(self, name, args, kwargs):
        ...
    


class FullEvalFormatter(Formatter):
    """A String Formatter that allows evaluation of simple expressions.
    
    Any time a format key is not found in the kwargs,
    it will be tried as an expression in the kwargs namespace.
    
    Note that this version allows slicing using [1:2], so you cannot specify
    a format string. Use :class:`EvalFormatter` to permit format strings.
    
    Examples
    --------
    ::

        In [1]: f = FullEvalFormatter()
        In [2]: f.format('{n//4}', n=8)
        Out[2]: '2'

        In [3]: f.format('{list(range(5))[2:4]}')
        Out[3]: '[2, 3]'

        In [4]: f.format('{3*2}')
        Out[4]: '6'
    """
    def vformat(self, format_string: str, args, kwargs) -> str:
        ...
    


class DollarFormatter(FullEvalFormatter):
    """Formatter allowing Itpl style $foo replacement, for names and attribute
    access only. Standard {foo} replacement also works, and allows full
    evaluation of its arguments.

    Examples
    --------
    ::

        In [1]: f = DollarFormatter()
        In [2]: f.format('{n//4}', n=8)
        Out[2]: '2'

        In [3]: f.format('23 * 76 is $result', result=23*76)
        Out[3]: '23 * 76 is 1748'

        In [4]: f.format('$a or {b}', a=1, b=2)
        Out[4]: '1 or 2'
    """
    _dollar_pattern_ignore_single_quote = ...
    def parse(self, fmt_string):
        ...
    
    def __repr__(self):
        ...
    


def compute_item_matrix(items, row_first=..., empty=..., *args, **kwargs):
    """Returns a nested list, and info to columnize items

    Parameters
    ----------
    items
        list of strings to columize
    row_first : (default False)
        Whether to compute columns for a row-first matrix instead of
        column-first (default).
    empty : (default None)
        default value to fill list if needed
    separator_size : int (default=2)
        How much characters will be used as a separation between each columns.
    displaywidth : int (default=80)
        The width of the area onto which the columns should enter

    Returns
    -------
    strings_matrix
        nested list of string, the outer most list contains as many list as
        rows, the innermost lists have each as many element as columns. If the
        total number of elements in `items` does not equal the product of
        rows*columns, the last element of some lists are filled with `None`.
    dict_info
        some info to make columnize easier:

        num_columns
          number of columns
        max_rows
          maximum number of rows (final number may be less)
        column_widths
          list of with of each columns
        optimal_separator_width
          best separator width between columns

    Examples
    --------
    ::

        In [1]: l = ['aaa','b','cc','d','eeeee','f','g','h','i','j','k','l']
        In [2]: list, info = compute_item_matrix(l, displaywidth=12)
        In [3]: list
        Out[3]: [['aaa', 'f', 'k'], ['b', 'g', 'l'], ['cc', 'h', None], ['d', 'i', None], ['eeeee', 'j', None]]
        In [4]: ideal = {'num_columns': 3, 'column_widths': [5, 1, 1], 'optimal_separator_width': 2, 'max_rows': 5}
        In [5]: all((info[k] == ideal[k] for k in ideal.keys()))
        Out[5]: True
    """
    ...

def columnize(items, row_first=..., separator=..., displaywidth=..., spread=...):
    """Transform a list of strings into a single string with columns.

    Parameters
    ----------
    items : sequence of strings
        The strings to process.
    row_first : (default False)
        Whether to compute columns for a row-first matrix instead of
        column-first (default).
    separator : str, optional [default is two spaces]
        The string that separates columns.
    displaywidth : int, optional [default is 80]
        Width of the display in number of characters.

    Returns
    -------
    The formatted string.
    """
    ...

def get_text_list(list_, last_sep=..., sep=..., wrap_item_with=...):
    """
    Return a string with a natural enumeration of items

    >>> get_text_list(['a', 'b', 'c', 'd'])
    'a, b, c and d'
    >>> get_text_list(['a', 'b', 'c'], ' or ')
    'a, b or c'
    >>> get_text_list(['a', 'b', 'c'], ', ')
    'a, b, c'
    >>> get_text_list(['a', 'b'], ' or ')
    'a or b'
    >>> get_text_list(['a'])
    'a'
    >>> get_text_list([])
    ''
    >>> get_text_list(['a', 'b'], wrap_item_with="`")
    '`a` and `b`'
    >>> get_text_list(['a', 'b', 'c', 'd'], " = ", sep=" + ")
    'a + b + c = d'
    """
    ...




from Cython import Tempita as Tempita
from .openmp_helpers import check_openmp_support as check_openmp_support
from Cython.Build import cythonize as cythonize
from .._min_dependencies import CYTHON_MIN_VERSION as CYTHON_MIN_VERSION
from ..externals._packaging.version import parse as parse
from .pre_build_helpers import basic_check_build as basic_check_build

# author: Andy Mueller, Gael Varoquaux
# license: BSD


import os as os
import sklearn as sklearn
import contextlib as contextlib


DEFAULT_ROOT: str = ...


def cythonize_extensions(extension):
    ...


def gen_from_templates(templates):
    ...



from typing import (
    Any,
    Dict,
    Generic,
    Iterable,
    List,
    MutableMapping,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from django.db.models.base import Model
from django.db.models.query import QuerySet

_T = TypeVar("_T", bound=Model)
_V = TypeVar("_V", bound=Model)
_M = TypeVar("_M", bound="BaseManager[Any]")

class BaseManager(QuerySet[_T]):
    creation_counter: int = ...
    auto_created: bool = ...
    use_in_migrations: bool = ...
    name: str = ...
    model: Type[_T] = ...
    db: str
    _db: Optional[str]
    def __init__(self) -> None: ...
    def deconstruct(
        self,
    ) -> Tuple[bool, str, None, Tuple[Any, ...], Dict[str, int]]: ...
    def check(self, **kwargs: Any) -> List[Any]: ...
    @classmethod
    def from_queryset(
        cls, queryset_class: Type[QuerySet[Any]], class_name: Optional[str] = ...
    ) -> Any: ...
    @classmethod
    def _get_queryset_methods(cls, queryset_class: type) -> Dict[str, Any]: ...
    def contribute_to_class(self, model: Type[Model], name: str) -> None: ...
    def db_manager(
        self: _M, using: Optional[str] = ..., hints: Optional[Dict[str, Model]] = ...
    ) -> _M: ...
    def get_queryset(self) -> QuerySet[_T]: ...

class Manager(BaseManager[_T]): ...

class RelatedManager(Manager[_T]):
    related_val: Tuple[int, ...]
    def add(self, *objs: Union[QuerySet[_T], _T], bulk: bool = ...) -> None: ...
    async def aadd(self, *objs: Union[QuerySet[_T], _T], bulk: bool = ...) -> None: ...
    def remove(self, *objs: Union[QuerySet[_T], _T], bulk: bool = ...) -> None: ...
    async def aremove(
        self, *objs: Union[QuerySet[_T], _T], bulk: bool = ...
    ) -> None: ...
    def set(
        self,
        objs: Union[QuerySet[_T], Iterable[_T]],
        *,
        bulk: bool = ...,
        clear: bool = ...,
    ) -> None: ...
    async def aset(
        self,
        objs: Union[QuerySet[_T], Iterable[_T]],
        *,
        bulk: bool = ...,
        clear: bool = ...,
    ) -> None: ...
    def clear(self) -> None: ...
    async def aclear(self) -> None: ...

class ManyToManyRelatedManager(Generic[_T, _V], Manager[_T]):
    through: Type[_V]
    def add(
        self,
        *objs: Union[QuerySet[_T], _T, _V],
        through_defaults: MutableMapping[str, Any] = ...,
    ) -> None: ...
    async def aadd(
        self,
        *objs: Union[QuerySet[_T], _T, _V],
        through_defaults: MutableMapping[str, Any] = ...,
    ) -> None: ...
    def remove(self, *objs: Union[QuerySet[_T], _T, _V]) -> None: ...
    async def aremove(self, *objs: Union[QuerySet[_T], _T, _V]) -> None: ...
    def set(
        self,
        objs: Union[QuerySet[_T], Iterable[_T]],
        *,
        clear: bool = ...,
        through_defaults: MutableMapping[str, Any] = ...,
    ) -> None: ...
    async def aset(
        self,
        objs: Union[QuerySet[_T], Iterable[_T]],
        *,
        clear: bool = ...,
        through_defaults: MutableMapping[str, Any] = ...,
    ) -> None: ...
    def clear(self) -> None: ...
    async def aclear(self) -> None: ...
    def create(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        through_defaults: Optional[MutableMapping[str, Any]] = ...,
        **kwargs: Any,
    ) -> _T: ...
    async def acreate(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        through_defaults: Optional[MutableMapping[str, Any]] = ...,
        **kwargs: Any,
    ) -> _T: ...
    def get_or_create(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        *,
        through_defaults: MutableMapping[str, Any] = ...,
        **kwargs: Any,
    ) -> Tuple[_T, bool]: ...
    async def aget_or_create(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        *,
        through_defaults: MutableMapping[str, Any] = ...,
        **kwargs: Any,
    ) -> Tuple[_T, bool]: ...
    def update_or_create(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        *,
        through_defaults: MutableMapping[str, Any] = ...,
        **kwargs: Any,
    ) -> Tuple[_T, bool]: ...
    async def aupdate_or_create(
        self,
        defaults: Optional[MutableMapping[str, Any]] = ...,
        *,
        through_defaults: MutableMapping[str, Any] = ...,
        **kwargs: Any,
    ) -> Tuple[_T, bool]: ...

class ManagerDescriptor:
    manager: Manager[Any] = ...
    def __init__(self, manager: Manager[Any]) -> None: ...
    def __get__(
        self, instance: Optional[Model], cls: Type[Model] = ...
    ) -> Manager[Any]: ...

class EmptyManager(Manager[Any]):
    def __init__(self, model: Type[Model]) -> None: ...



from typing import Literal, Sequence


# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from copy import deepcopy

from ..util import logger
from ._color_dict import _color_dict
from .color_space import (
    _hex_to_rgba,
    _rgb_to_hex,
    _rgb_to_hsv,  # noqa
    _hsv_to_rgb,
    _rgb_to_lab,
    _lab_to_rgb,
)  # noqa

###############################################################################
# User-friendliness helpers

def _string_to_rgb(color): ...
def _user_to_rgba(color, expand=True, clip=False): ...
def _array_clip_val(val): ...

###############################################################################
# Color Array

class ColorArray(object):
    def __init__(
        self,
        color: str | tuple | Sequence = ...,
        alpha: None | float = None,
        clip: bool = False,
        color_space: Literal["hsv", "rgb"] = "rgb",
    ): ...

    ###########################################################################
    # Builtins and utilities
    def copy(self): ...
    @classmethod
    def _name(cls): ...
    def __len__(self): ...
    def __repr__(self): ...
    def __eq__(self, other): ...

    ###########################################################################
    def __getitem__(self, item): ...
    def __setitem__(self, item, value): ...
    def extend(self, colors: ColorArray): ...

    # RGB(A)
    @property
    def rgba(self): ...
    @rgba.setter
    def rgba(self, val): ...
    @property
    def rgb(self): ...
    @rgb.setter
    def rgb(self, val): ...
    @property
    def RGBA(self): ...
    @RGBA.setter
    def RGBA(self, val): ...
    @property
    def RGB(self): ...
    @RGB.setter
    def RGB(self, val): ...
    @property
    def alpha(self): ...
    @alpha.setter
    def alpha(self, val): ...

    ###########################################################################
    # HEX
    @property
    def hex(self): ...
    @hex.setter
    def hex(self, val): ...

    ###########################################################################
    # HSV
    @property
    def hsv(self): ...
    @hsv.setter
    def hsv(self, val): ...
    @property
    def _hsv(self): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, val): ...
    def lighter(self, dv: float = 0.1, copy: bool = True) -> ColorArray: ...
    def darker(self, dv: float = 0.1, copy: bool = True) -> ColorArray: ...

    ###########################################################################
    # Lab
    @property
    def lab(self): ...
    @lab.setter
    def lab(self, val): ...

class Color(ColorArray):
    def __init__(
        self,
        color: str | tuple = "black",
        alpha: None | float = None,
        clip: bool = False,
    ): ...
    @ColorArray.rgba.getter
    def rgba(self): ...
    @ColorArray.rgb.getter
    def rgb(self): ...
    @ColorArray.RGBA.getter
    def RGBA(self): ...
    @ColorArray.RGB.getter
    def RGB(self): ...
    @ColorArray.alpha.getter
    def alpha(self): ...
    @ColorArray.hex.getter
    def hex(self): ...
    @ColorArray.hsv.getter
    def hsv(self): ...
    @ColorArray.value.getter
    def value(self): ...
    @ColorArray.lab.getter
    def lab(self): ...
    @property
    def is_blank(self): ...
    def __repr__(self): ...



from collections.abc import Iterable

import warnings

from ..classes.graph import Graph
from ..utils import open_file

__all__ = ["read_pajek", "parse_pajek", "generate_pajek", "write_pajek"]

def generate_pajek(G: Graph): ...
def write_pajek(G: Graph, path, encoding="UTF-8"): ...
def read_pajek(path, encoding="UTF-8"): ...
def parse_pajek(lines: str | Iterable): ...
def make_qstr(t): ...



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# glumpy is an OpenGL framework for the fast visualization of numpy arrays.
# Copyright (C) 2009-2011  Nicolas P. Rougier. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY NICOLAS P. ROUGIER ''AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL NICOLAS P. ROUGIER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of Nicolas P. Rougier.
# -----------------------------------------------------------------------------

import math
import numpy as np
from inspect import cleandoc
from itertools import product

class SpatialFilter:
    def __init__(self, radius=1): ...
    def weight(self, x): ...
    def kernel(self, size=...): ...
    def call_code(self, index): ...

class Linear(SpatialFilter):
    def weight(self, x): ...

class Hanning(SpatialFilter):
    def weight(self, x): ...

class Hamming(SpatialFilter):
    def weight(self, x): ...

class Hermite(SpatialFilter):
    def weight(self, x): ...

class Quadric(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Bicubic(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Kaiser(SpatialFilter):
    def __init__(self, b=6.33): ...
    def bessel_i0(self, x): ...
    def weight(self, x): ...

class CatRom(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Mitchell(SpatialFilter):
    def __init__(self, b=..., c=...): ...
    def weight(self, x): ...

class Spline16(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Spline36(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Gaussian(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Bessel(SpatialFilter):
    def __init__(self): ...
    def besj(self, x: int, n: int): ...
    def weight(self, x): ...

class Sinc(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Lanczos(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

class Blackman(SpatialFilter):
    def __init__(self): ...
    def weight(self, x): ...

def generate_filter_code(radius): ...
def main(): ...



from .connectivity import (
    average_node_connectivity as average_node_connectivity,
    local_node_connectivity as local_node_connectivity,
    node_connectivity as node_connectivity,
    local_edge_connectivity as local_edge_connectivity,
    edge_connectivity as edge_connectivity,
    all_pairs_node_connectivity as all_pairs_node_connectivity,
)
from .cuts import (
    minimum_st_node_cut as minimum_st_node_cut,
    minimum_node_cut as minimum_node_cut,
    minimum_st_edge_cut as minimum_st_edge_cut,
    minimum_edge_cut as minimum_edge_cut,
)
from .edge_augmentation import (
    k_edge_augmentation as k_edge_augmentation,
    is_k_edge_connected as is_k_edge_connected,
    is_locally_k_edge_connected as is_locally_k_edge_connected,
)
from .edge_kcomponents import (
    k_edge_components as k_edge_components,
    k_edge_subgraphs as k_edge_subgraphs,
    bridge_components as bridge_components,
    EdgeComponentAuxGraph as EdgeComponentAuxGraph,
)
from .disjoint_paths import (
    edge_disjoint_paths as edge_disjoint_paths,
    node_disjoint_paths as node_disjoint_paths,
)
from .kcomponents import k_components as k_components
from .kcutsets import all_node_cuts as all_node_cuts
from .stoerwagner import stoer_wagner as stoer_wagner
from .utils import (
    build_auxiliary_node_connectivity as build_auxiliary_node_connectivity,
    build_auxiliary_edge_connectivity as build_auxiliary_edge_connectivity,
)



from typing import Literal
import numpy as np

from .._shared.utils import _supported_float_type, check_nD

class FeatureDetector(object):
    def __init__(self): ...
    def detect(self, image): ...

class DescriptorExtractor(object):
    def __init__(self): ...
    def extract(self, image, keypoints): ...

def plot_matches(
    ax,
    image1,
    image2,
    keypoints1,
    keypoints2,
    matches,
    keypoints_color="k",
    matches_color=None,
    only_matches: bool = False,
    alignment: Literal["horizontal", "vertical"] = "horizontal",
): ...
def _prepare_grayscale_input_2D(image): ...
def _prepare_grayscale_input_nD(image): ...
def _mask_border_keypoints(image_shape, keypoints, distance): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.cluster._hierarchy, version: unspecified
import typing
import builtins as _mod_builtins

class Heap(_mod_builtins.object):
    'Binary heap.\n\n    Heap stores values and keys. Values are passed explicitly, whereas keys\n    are assigned implicitly to natural numbers (from 0 to n - 1).\n\n    The supported operations (all have O(log n) time complexity):\n\n        * Return the current minimum value and the corresponding key.\n        * Remove the current minimum value.\n        * Change the value of the given key. Note that the key must be still\n          in the heap.\n\n    The heap is stored as an array, where children of parent i have indices\n    2 * i + 1 and 2 * i + 2. All public methods are based on  `sift_down` and\n    `sift_up` methods, which restore the heap property by moving an element\n    down or up in the heap.\n    '
    def __init__(self, *args, **kwargs) -> None:
        'Binary heap.\n\n    Heap stores values and keys. Values are passed explicitly, whereas keys\n    are assigned implicitly to natural numbers (from 0 to n - 1).\n\n    The supported operations (all have O(log n) time complexity):\n\n        * Return the current minimum value and the corresponding key.\n        * Remove the current minimum value.\n        * Change the value of the given key. Note that the key must be still\n          in the heap.\n\n    The heap is stored as an array, where children of parent i have indices\n    2 * i + 1 and 2 * i + 2. All public methods are based on  `sift_down` and\n    `sift_up` methods, which restore the heap property by moving an element\n    down or up in the heap.\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def change_value(self) -> typing.Any:
        ...
    
    def get_min(self) -> typing.Any:
        ...
    
    def remove_min(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class LinkageUnionFind(_mod_builtins.object):
    'Structure for fast cluster labeling in unsorted dendrogram.'
    def __init__(self, *args, **kwargs) -> None:
        'Structure for fast cluster labeling in unsorted dendrogram.'
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

def __pyx_unpickle_Heap() -> typing.Any:
    ...

def __pyx_unpickle_LinkageUnionFind() -> typing.Any:
    ...

__test__: dict
def calculate_cluster_sizes() -> typing.Any:
    '\n    Calculate the size of each cluster. The result is the fourth column of\n    the linkage matrix.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix. The fourth column can be empty.\n    cs : ndarray\n        The array to store the sizes.\n    n : ndarray\n        The number of observations.\n    '
    ...

def cluster_dist() -> typing.Any:
    "\n    Form flat clusters by distance criterion.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    T : ndarray\n        The array to store the cluster numbers. The i'th observation belongs to\n        cluster `T[i]`.\n    cutoff : double\n        Clusters are formed when distances are less than or equal to `cutoff`.\n    n : int\n        The number of observations.\n    "
    ...

def cluster_in() -> typing.Any:
    "\n    Form flat clusters by inconsistent criterion.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    R : ndarray\n        The inconsistent matrix.\n    T : ndarray\n        The array to store the cluster numbers. The i'th observation belongs to\n        cluster `T[i]`.\n    cutoff : double\n        Clusters are formed when the inconsistent values are less than or\n        or equal to `cutoff`.\n    n : int\n        The number of observations.\n    "
    ...

def cluster_maxclust_dist() -> typing.Any:
    "\n    Form flat clusters by maxclust criterion.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    T : ndarray\n        The array to store the cluster numbers. The i'th observation belongs to\n        cluster `T[i]`.\n    n : int\n        The number of observations.\n    mc : int\n        The maximum number of clusters.\n    "
    ...

def cluster_maxclust_monocrit() -> typing.Any:
    "\n    Form flat clusters by maxclust_monocrit criterion.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    MC : ndarray\n        The monotonic criterion array.\n    T : ndarray\n        The array to store the cluster numbers. The i'th observation belongs to\n        cluster `T[i]`.\n    n : int\n        The number of observations.\n    max_nc : int\n        The maximum number of clusters.\n    "
    ...

def cluster_monocrit() -> typing.Any:
    "\n    Form flat clusters by monocrit criterion.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    MC : ndarray\n        The monotonic criterion array.\n    T : ndarray\n        The array to store the cluster numbers. The i'th observation belongs to\n        cluster `T[i]`.\n    cutoff : double\n        Clusters are formed when the MC values are less than or equal to\n        `cutoff`.\n    n : int\n        The number of observations.\n    "
    ...

def cophenetic_distances() -> typing.Any:
    '\n    Calculate the cophenetic distances between each observation\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    d : ndarray\n        The condensed matrix to store the cophenetic distances.\n    n : int\n        The number of observations.\n    '
    ...

def fast_linkage() -> typing.Any:
    'Perform hierarchy clustering.\n\n    It implements "Generic Clustering Algorithm" from [1]. The worst case\n    time complexity is O(N^3), but the best case time complexity is O(N^2) and\n    it usually works quite close to the best case.\n\n    Parameters\n    ----------\n    dists : ndarray\n        A condensed matrix stores the pairwise distances of the observations.\n    n : int\n        The number of observations.\n    method : int\n        The linkage method. 0: single 1: complete 2: average 3: centroid\n        4: median 5: ward 6: weighted\n\n    Returns\n    -------\n    Z : ndarray, shape (n - 1, 4)\n        Computed linkage matrix.\n\n    References\n    ----------\n    .. [1] Daniel Mullner, "Modern hierarchical, agglomerative clustering\n       algorithms", :arXiv:`1109.2378v1`.\n    '
    ...

def get_max_Rfield_for_each_cluster() -> typing.Any:
    "\n    Get the maximum statistic for each non-singleton cluster. For the i'th\n    non-singleton cluster, max_rfs[i] = max{R[j, rf] j is a descendent of i}.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    R : ndarray\n        The R matrix.\n    max_rfs : ndarray\n        The array to store the result.\n    n : int\n        The number of observations.\n    rf : int\n        Indicate which column of `R` is used.\n    "
    ...

def get_max_dist_for_each_cluster() -> typing.Any:
    '\n    Get the maximum inconsistency coefficient for each non-singleton cluster.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    MD : ndarray\n        The array to store the result.\n    n : int\n        The number of observations.\n    '
    ...

def inconsistent() -> typing.Any:
    '\n    Calculate the inconsistency statistics.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    R : ndarray\n        A (n - 1) x 4 matrix to store the result. The inconsistency statistics\n        `R[i]` are calculated over `d` levels below cluster i. `R[i, 0]` is the\n        mean of distances. `R[i, 1]` is the standard deviation of distances.\n        `R[i, 2]` is the number of clusters included. `R[i, 3]` is the\n        inconsistency coefficient.\n\n        .. math:: \\frac{\\mathtt{Z[i,2]}-\\mathtt{R[i,0]}} {R[i,1]}\n\n    n : int\n        The number of observations.\n    d : int\n        The number of levels included in calculation below a node.\n    '
    ...

def leaders() -> typing.Any:
    '\n    Find the leader (root) of each flat cluster.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    T : ndarray\n        The flat clusters assignment returned by `fcluster` or `fclusterdata`.\n    L : ndarray\n        `L` and `M` store the result. The leader of flat cluster `L[i]` is\n        node `M[i]`.\n    M : ndarray\n        `L` and `M` store the result. The leader of flat cluster `L[i]` is\n        node `M[i]`.\n    nc : int\n        The number of flat clusters.\n    n : int\n        The number of observations.\n\n    Returns\n    -------\n    err_node : int\n        Found that `T` is invalid when examining node `err_node`.\n        `-1` indicates success.\n    '
    ...

def linkage() -> typing.Any:
    '\n    Perform hierarchy clustering.\n\n    Parameters\n    ----------\n    dists : ndarray\n        A condensed matrix stores the pairwise distances of the observations.\n    n : int\n        The number of observations.\n    method : int\n        The linkage method. 0: single 1: complete 2: average 3: centroid\n        4: median 5: ward 6: weighted\n\n    Returns\n    -------\n    Z : ndarray, shape (n - 1, 4)\n        Computed linkage matrix.\n    '
    ...

def mst_single_linkage() -> typing.Any:
    'Perform hierarchy clustering using MST algorithm for single linkage.\n\n    Parameters\n    ----------\n    dists : ndarray\n        A condensed matrix stores the pairwise distances of the observations.\n    n : int\n        The number of observations.\n\n    Returns\n    -------\n    Z : ndarray, shape (n - 1, 4)\n        Computed linkage matrix.\n    '
    ...

def nn_chain() -> typing.Any:
    'Perform hierarchy clustering using nearest-neighbor chain algorithm.\n\n    Parameters\n    ----------\n    dists : ndarray\n        A condensed matrix stores the pairwise distances of the observations.\n    n : int\n        The number of observations.\n    method : int\n        The linkage method. 0: single 1: complete 2: average 3: centroid\n        4: median 5: ward 6: weighted\n\n    Returns\n    -------\n    Z : ndarray, shape (n - 1, 4)\n        Computed linkage matrix.\n    '
    ...

def prelist() -> typing.Any:
    '\n    Perform a pre-order traversal on the linkage tree and get a list of ids\n    of the leaves.\n\n    Parameters\n    ----------\n    Z : ndarray\n        The linkage matrix.\n    members : ndarray\n        The array to store the result.\n    n : int\n        The number of observations.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-

from itertools import permutations
import numpy as np

from collections import OrderedDict

class Triangulation(object):
    def __init__(self, pts: ArrayLike, edges: ArrayLike): ...
    def _normalize(self): ...
    def _initialize(self): ...
    def triangulate(self): ...
    def _finalize(self): ...
    def _edge_event(self, i, j): ...
    def _find_cut_triangle(self, edge): ...
    def _edge_in_front(self, edge): ...
    def _edge_opposite_point(self, tri, i): ...
    def _adjacent_tri(self, edge, i): ...
    def _tri_from_edge(self, edge): ...
    def _edges_in_tri_except(self, tri, edge): ...
    def _edge_below_front(self, edge, front_index): ...
    def _is_constraining_edge(self, edge): ...
    def _intersected_edge(self, edges, cut_edge): ...
    def _find_edge_intersections(self): ...
    def _split_intersecting_edges(self): ...
    def _merge_duplicate_points(self): ...
    def _distances_from_line(self, edge, points): ...
    def _projection(self, a, b, c): ...
    def _cosine(self, A, B, C): ...
    def _iscounterclockwise(self, a, b, c): ...
    def _edges_intersect(self, edge1, edge2): ...
    def _intersect_edge_arrays(self, lines1, lines2): ...
    def _orientation(self, edge, point): ...
    def _add_tri(self, a, b, c): ...
    def _remove_tri(self, a, b, c): ...

def _triangulate_python(vertices_2d, segments): ...
def _triangulate_cpp(vertices_2d, segments): ...
def triangulate(vertices: ArrayLike) -> tuple[ArrayLike, ArrayLike]: ...



from typing import Mapping
from ...classes.graph import Graph

__all__ = ["hits", "hits_numpy", "hits_scipy", "authority_matrix", "hub_matrix"]

def hits(
    G: Graph,
    max_iter=100,
    tol: float = 1.0e-8,
    nstart: Mapping | None = None,
    normalized=True,
): ...
def authority_matrix(G: Graph, nodelist=None): ...
def hub_matrix(G: Graph, nodelist=None): ...
def hits_numpy(G: Graph, normalized=True): ...
def hits_scipy(
    G: Graph,
    max_iter=100,
    tol: float = 1.0e-6,
    nstart: Mapping | None = None,
    normalized=True,
): ...



from typing import Callable, ClassVar, Literal, TypeVar
from scipy.stats import spearmanr as spearmanr
from scipy import interpolate as interpolate
from .base import BaseEstimator, TransformerMixin, RegressorMixin
from numpy import ndarray
from numbers import Real as Real
from .utils import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
)
from .utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ._typing import ArrayLike, Float

IsotonicRegression_Self = TypeVar("IsotonicRegression_Self", bound="IsotonicRegression")

# Authors: Fabian Pedregosa <fabian@fseoane.net>
#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Nelle Varoquaux <nelle.varoquaux@gmail.com>
# License: BSD 3 clause

import numpy as np
import warnings
import math


__all__ = ["check_increasing", "isotonic_regression", "IsotonicRegression"]


def check_increasing(x: ArrayLike, y: ArrayLike) -> bool:
    ...


def isotonic_regression(
    y: ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    y_min: None | Float = None,
    y_max: None | Float = None,
    increasing: bool = True
) -> ndarray | list[float]:
    ...


class IsotonicRegression(RegressorMixin, TransformerMixin, BaseEstimator):
    increasing_: bool = ...
    f_: Callable = ...
    y_thresholds_: ndarray = ...
    X_thresholds_: ndarray = ...
    X_max_: float = ...
    X_min_: float = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        y_min: None | Float = None,
        y_max: None | Float = None,
        increasing: str | bool = True,
        out_of_bounds: Literal["nan", "clip", "raise", "nan"] = "nan"
    ) -> None:
        ...

    def fit(
        self: IsotonicRegression_Self,
        X: ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> IsotonicRegression_Self:
        ...

    def transform(self, T: ArrayLike) -> ndarray:
        ...

    def predict(self, T: ArrayLike) -> ndarray:
        ...

    # We implement get_feature_names_out here instead of using
    # `ClassNamePrefixFeaturesOutMixin`` because `input_features` are ignored.
    # `input_features` are ignored because `IsotonicRegression` accepts 1d
    # arrays and the semantics of `feature_names_in_` are not clear for 1d arrays.
    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def __getstate__(self):
        ...

    def __setstate__(self, state):
        ...



from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy import linalg as linalg
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..utils.extmath import (
    fast_logdet as fast_logdet,
    randomized_svd as randomized_svd,
    squared_norm as squared_norm,
)
from numbers import Integral as Integral, Real as Real
from math import sqrt as sqrt, log as log
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import Int, Float, ArrayLike, MatrixLike
from ..utils import check_random_state as check_random_state
from ..utils.validation import check_is_fitted as check_is_fitted

FactorAnalysis_Self = TypeVar("FactorAnalysis_Self", bound="FactorAnalysis")


# Author: Christian Osendorfer <osendorf@gmail.com>
#         Alexandre Gramfort <alexandre.gramfort@inria.fr>
#         Denis A. Engemann <denis-alexander.engemann@inria.fr>

# License: BSD3

import warnings
import numpy as np


class FactorAnalysis(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    mean_: ndarray = ...
    n_iter_: int = ...
    noise_variance_: ndarray = ...
    loglike_: list = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        tol: Float = 1e-2,
        copy: bool = True,
        max_iter: Int = 1000,
        noise_variance_init: None | ArrayLike = None,
        svd_method: Literal["lapack", "randomized", "randomized"] = "randomized",
        iterated_power: Int = 3,
        rotation: None | Literal["varimax", "quartimax"] = None,
        random_state: None | RandomState | int = 0,
    ) -> None:
        ...

    def fit(
        self: FactorAnalysis_Self, X: MatrixLike, y: Any = None
    ) -> FactorAnalysis_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def get_covariance(self) -> ndarray:
        ...

    def get_precision(self) -> ndarray:
        ...

    def score_samples(self, X: ArrayLike) -> ndarray:
        ...

    def score(self, X: ArrayLike, y: Any = None) -> float:
        ...
